/*! ****************************************************************************
Copyright (c) Microblink. All rights reserved.

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.
***************************************************************************** */
(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
typeof define === 'function' && define.amd ? define(['exports'], factory) :
(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.BlinkCardSDK = {}));
}(this, (function (exports) { 'use strict';

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
let nextMessageID = 0;
function getNextMessageID() {
    const msgId = nextMessageID;
    nextMessageID = nextMessageID + 1;
    return msgId;
}
class BaseRequestMessage {
    constructor(action) {
        this.action = action;
        this.messageID = getNextMessageID();
    }
}
class InitMessage extends BaseRequestMessage {
    constructor(wasmLoadSettings, userId) {
        super(InitMessage.action);
        this.wasmModuleName = wasmLoadSettings.wasmModuleName;
        this.licenseKey = wasmLoadSettings.licenseKey;
        this.userId = userId;
        this.registerLoadCallback = wasmLoadSettings.loadProgressCallback !== null;
        this.allowHelloMessage = wasmLoadSettings.allowHelloMessage;
        this.engineLocation = wasmLoadSettings.engineLocation;
        this.wasmType = wasmLoadSettings.wasmType;
        this.wasmFlavor = wasmLoadSettings.wasmFlavor;
        this.numberOfWorkers = wasmLoadSettings.numberOfWorkers;
    }
}
InitMessage.action = "init";
var ParameterType;
(function (ParameterType) {
    ParameterType[ParameterType["Any"] = 0] = "Any";
    ParameterType[ParameterType["Recognizer"] = 1] = "Recognizer";
    ParameterType[ParameterType["RecognizerSettings"] = 2] = "RecognizerSettings";
    ParameterType[ParameterType["Callback"] = 3] = "Callback";
})(ParameterType || (ParameterType = {}));
class CreateNewRecognizer extends BaseRequestMessage {
    constructor(className, params) {
        super(CreateNewRecognizer.action);
        this.className = className;
        this.params = params;
    }
}
CreateNewRecognizer.action = "createNewNativeObject";
class CreateRecognizerRunner extends BaseRequestMessage {
    constructor(recognizerHandles, allowMultipleResults, registeredMetadataCallbacks) {
        super(CreateRecognizerRunner.action);
        this.recognizerHandles = recognizerHandles;
        this.allowMultipleResults = allowMultipleResults;
        this.registeredMetadataCallbacks = registeredMetadataCallbacks;
    }
}
CreateRecognizerRunner.action = "createRecognizerRunner";
class ReconfigureRecognizerRunner extends BaseRequestMessage {
    constructor(recognizerHandles, allowMultipleResults) {
        super(ReconfigureRecognizerRunner.action);
        this.recognizerHandles = recognizerHandles;
        this.allowMultipleResults = allowMultipleResults;
    }
}
ReconfigureRecognizerRunner.action = "reconfigureRecognizerRunner";
class DeleteRecognizerRunner extends BaseRequestMessage {
    constructor() {
        super(DeleteRecognizerRunner.action);
    }
}
DeleteRecognizerRunner.action = "deleteRecognizerRunner";
class InvokeObjectMethod extends BaseRequestMessage {
    constructor(objectHandle, methodName, params) {
        super(InvokeObjectMethod.action);
        this.objectHandle = objectHandle;
        this.methodName = methodName;
        this.params = params;
    }
}
InvokeObjectMethod.action = "invokeObject";
class ProcessImage extends BaseRequestMessage {
    constructor(image) {
        super(ProcessImage.action);
        this.frame = image;
    }
    getTransferrables() {
        return [this.frame.imageData.data.buffer];
    }
}
ProcessImage.action = "processImage";
class ResetRecognizers extends BaseRequestMessage {
    constructor(hardReset) {
        super(ResetRecognizers.action);
        this.hardReset = hardReset;
    }
}
ResetRecognizers.action = "resetRecognizers";
class RegisteredMetadataCallbacks {
    constructor() {
        this.onDebugText = false;
        this.onDetectionFailed = false;
        this.onQuadDetection = false;
        this.onPointsDetection = false;
        this.onFirstSideResult = false;
        this.onGlare = false;
    }
}
class RegisterMetadataCallbacks extends BaseRequestMessage {
    constructor(registeredMetadataCallbacks) {
        super(RegisterMetadataCallbacks.action);
        this.registeredMetadataCallbacks = registeredMetadataCallbacks;
    }
}
RegisterMetadataCallbacks.action = "registerMetadataCallbacks";
class SetDetectionOnly extends BaseRequestMessage {
    constructor(detectionOnlyMode) {
        super(SetDetectionOnly.action);
        this.detectionOnlyMode = detectionOnlyMode;
    }
}
SetDetectionOnly.action = "setDetectionOnly";
class SetCameraPreviewMirrored extends BaseRequestMessage {
    constructor(cameraPreviewMirrored) {
        super(SetCameraPreviewMirrored.action);
        this.cameraPreviewMirrored = cameraPreviewMirrored;
    }
}
SetCameraPreviewMirrored.action = "setCameraPreviewMirrored";
class GetProductIntegrationInfo extends BaseRequestMessage {
    constructor(userId) {
        super(GetProductIntegrationInfo.action);
        this.userId = userId;
    }
}
GetProductIntegrationInfo.action = "getProductIntegrationInfo";
// ===================================== /
// Metadata callback messages
// ===================================== /
var MetadataCallback;
(function (MetadataCallback) {
    MetadataCallback[MetadataCallback["onDebugText"] = 0] = "onDebugText";
    MetadataCallback[MetadataCallback["onDetectionFailed"] = 1] = "onDetectionFailed";
    MetadataCallback[MetadataCallback["onQuadDetection"] = 2] = "onQuadDetection";
    MetadataCallback[MetadataCallback["onPointsDetection"] = 3] = "onPointsDetection";
    MetadataCallback[MetadataCallback["onFirstSideResult"] = 4] = "onFirstSideResult";
    MetadataCallback[MetadataCallback["clearTimeoutCallback"] = 5] = "clearTimeoutCallback";
    MetadataCallback[MetadataCallback["onGlare"] = 6] = "onGlare";
    MetadataCallback[MetadataCallback["recognizerCallback"] = 7] = "recognizerCallback";
})(MetadataCallback || (MetadataCallback = {}));
/* eslint-enable @typescript-eslint/no-explicit-any */

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (WasmType) {
    /**
     * The WASM that will be loaded will be most compatible with all browsers that
     * support the WASM, but will lack features that could be used to improve performance.
     */
    WasmType["Basic"] = "BASIC";
    /**
     * The WASM that will be loaded will be built with advanced WASM features, such as
     * bulk memory, SIMD, non-trapping floating point and sign extension. Such WASM can only
     * be executed in browsers that support those features. Attempting to run this
     * WASM in a non-compatible browser will crash your app.
     */
    WasmType["Advanced"] = "ADVANCED";
    /**
     * The WASM that will be loaded will be build with advanced WASM features, just
     * like above. Additionally, it will be also built with support for multi-threaded
     * processing. This feature requires a browser with support for both advanced WASM
     * features and `SharedArrayBuffer`
     */
    WasmType["AdvancedWithThreads"] = "ADVANCED_WITH_THREADS";
})(exports.WasmType || (exports.WasmType = {}));

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
const defaultWasmModuleName = "BlinkCardWasmSDK";
function defaultWasmFlavor() {
    return "";
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
/* eslint-disable @typescript-eslint/no-explicit-any,
                  @typescript-eslint/no-unsafe-assignment */
class SDKError extends Error {
    constructor(error, details) {
        super();
        if (!error.code || !error.message) {
            throw new Error("Instance of SDKError is required to have code and message.");
        }
        this.message = error.message;
        this.code = error.code;
        if (error instanceof SerializableSDKError && error.details && "message" in error.details) {
            const errorObj = new Error(error.details.message);
            if ("stack" in error.details) {
                errorObj.stack = error.details.stack;
            }
            this.details = errorObj;
        }
        else {
            this.details = details;
        }
    }
}
class SerializableSDKError {
    constructor(error, details) {
        if (!error.code || !error.message) {
            throw new Error("Instance of SDKError is required to have code and message.");
        }
        this.message = error.message;
        this.code = error.code;
        if (details instanceof Error) {
            this.details = new CustomError(details);
        }
        else {
            this.details = details;
        }
    }
}
class CustomError {
    constructor(error) {
        this.message = error.message;
        this.name = error.name;
        this.stack = error.stack;
    }
}

/* eslint-disable @typescript-eslint/no-unsafe-argument */
function defaultEventHandler(resolve, reject) {
    return (msg) => {
        const resultMsg = msg;
        if (resultMsg.success) {
            resolve();
        }
        else {
            if (typeof resultMsg.error === "string" || resultMsg.error === null) {
                reject(resultMsg.error);
            }
            else {
                reject(new SDKError(resultMsg.error));
            }
        }
    };
}
function defaultResultEventHandler(successResolver, reject) {
    return (msg) => {
        const resultMsg = msg;
        if (resultMsg.success) {
            successResolver(msg);
        }
        else {
            if (typeof resultMsg.error === "string" || resultMsg.error === null) {
                reject(resultMsg.error);
            }
            else {
                reject(new SDKError(resultMsg.error));
            }
        }
    };
}
/* eslint-disable @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-assignment */
function wrapParameters(params) {
    // convert params
    const wrappedPrameters = [];
    for (let param of params) {
        let paramType = ParameterType.Any;
        if (param instanceof RemoteRecognizer) {
            paramType = ParameterType.Recognizer;
            param = param.getRemoteObjectHandle();
        }
        wrappedPrameters.push({
            parameter: param,
            type: paramType
        });
    }
    return wrappedPrameters;
}
/* eslint-enable @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-assignment */
class RemoteRecognizer {
    /* eslint-enable lines-between-class-members */
    constructor(wasmWorker, recognizerName, remoteObjHandle) {
        this.wasmSDKWorker = wasmWorker;
        this.objectHandle = remoteObjHandle;
        this.recognizerName = recognizerName;
        this.callbacks = new Map();
    }
    /* eslint-enable @typescript-eslint/ban-types */
    getRemoteObjectHandle() {
        return this.objectHandle;
    }
    currentSettings() {
        return new Promise((resolve, reject) => {
            if (this.objectHandle < 0) {
                reject("Invalid object handle: " + this.objectHandle.toString());
                return;
            }
            const msg = new InvokeObjectMethod(this.objectHandle, "currentSettings", []);
            const handler = defaultResultEventHandler((msg) => {
                resolve(msg.result);
            }, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
    toSignedJSON() {
        {
            // native module does not support toSignedJSON method
            return Promise.resolve(null);
        }
    }
    clearAllCallbacks() {
        this.callbacks.clear();
        this.wasmSDKWorker.unregisterRecognizerCallbacks(this.objectHandle);
    }
    /* eslint-disable @typescript-eslint/no-explicit-any,
                      @typescript-eslint/no-unsafe-assignment,
                      @typescript-eslint/no-unsafe-member-access,
                      @typescript-eslint/no-unsafe-return
    */
    // convert each function member into wrapped parameter, containing address where callback needs to be delivered
    removeFunctions(settings) {
        // clear any existing callbacks
        this.clearAllCallbacks();
        const keys = Object.keys(settings);
        let needsRegistering = false;
        for (const key of keys) {
            const data = settings[key];
            if (typeof data === "function") {
                this.callbacks.set(key, data);
                const wrappedFunction = {
                    parameter: {
                        recognizerHandle: this.objectHandle,
                        callbackName: key
                    },
                    type: ParameterType.Callback
                };
                settings[key] = wrappedFunction;
                needsRegistering = true;
            }
        }
        if (needsRegistering) {
            this.wasmSDKWorker.registerRecognizerCallbacks(this.objectHandle, this);
        }
        return settings;
    }
    /* eslint-enable @typescript-eslint/no-explicit-any,
                     @typescript-eslint/no-unsafe-assignment,
                     @typescript-eslint/no-unsafe-member-access,
                     @typescript-eslint/no-unsafe-return
    */
    updateSettings(newSettings) {
        return new Promise((resolve, reject) => {
            if (this.objectHandle < 0) {
                reject("Invalid object handle: " + this.objectHandle.toString());
                return;
            }
            /* eslint-disable @typescript-eslint/no-unsafe-assignment */
            const msg = new InvokeObjectMethod(this.objectHandle, "updateSettings", [
                {
                    parameter: this.removeFunctions(newSettings),
                    type: ParameterType.RecognizerSettings
                }
            ]);
            /* eslint-enable @typescript-eslint/no-unsafe-assignment */
            const handler = defaultEventHandler(resolve, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
    /* eslint-disable @typescript-eslint/no-explicit-any */
    invokeCallback(callbackName, args) {
        const callback = this.callbacks.get(callbackName);
        if (callback !== undefined) {
            callback(...args);
        }
        else {
            console.warn("Cannot find callback", callbackName);
        }
    }
    /* eslint-enable @typescript-eslint/no-explicit-any */
    getResult() {
        return new Promise((resolve, reject) => {
            if (this.objectHandle < 0) {
                reject("Invalid object handle: " + this.objectHandle.toString());
                return;
            }
            const msg = new InvokeObjectMethod(this.objectHandle, "getResult", []);
            const handler = defaultResultEventHandler((msg) => {
                resolve(msg.result);
            }, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
    delete() {
        return new Promise((resolve, reject) => {
            if (this.objectHandle < 0) {
                reject("Invalid object handle: " + this.objectHandle.toString());
                return;
            }
            this.clearAllCallbacks();
            const msg = new InvokeObjectMethod(this.objectHandle, "delete", []);
            const handler = defaultEventHandler(() => {
                this.objectHandle = -1;
                resolve();
            }, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
}
function createRegisteredCallbacks(metadataCallbacks) {
    const msg = new RegisteredMetadataCallbacks();
    // https://stackoverflow.com/a/20093686/213057
    msg.onDebugText = !!metadataCallbacks.onDebugText;
    msg.onDetectionFailed = !!metadataCallbacks.onDetectionFailed;
    msg.onPointsDetection = !!metadataCallbacks.onPointsDetection;
    msg.onQuadDetection = !!metadataCallbacks.onQuadDetection;
    msg.onFirstSideResult = !!metadataCallbacks.onFirstSideResult;
    return msg;
}
class RemoteRecognizerRunner {
    constructor(wasmWorker, recognizers) {
        this.deleted = false;
        this.wasmSDKWorker = wasmWorker;
        this.recognizers = recognizers;
    }
    processImage(image) {
        return new Promise((resolve, reject) => {
            if (this.deleted) {
                reject("Recognizer runner is deleted. It cannot be used anymore!");
                return;
            }
            const msg = new ProcessImage(image);
            const handler = defaultResultEventHandler((response) => {
                const state = response.recognitionState;
                resolve(state);
            }, reject);
            this.wasmSDKWorker.postTransferrableMessage(msg, handler);
        });
    }
    reconfigureRecognizers(recognizers, allowMultipleResults) {
        return new Promise((resolve, reject) => {
            if (this.deleted) {
                reject("Recognizer runner is deleted. It cannot be used anymore!");
                return;
            }
            const recognizerHandles = getRecognizerHandles(recognizers);
            const msg = new ReconfigureRecognizerRunner(recognizerHandles, allowMultipleResults);
            const handler = defaultEventHandler(resolve, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
            this.recognizers = recognizers;
        });
    }
    setMetadataCallbacks(metadataCallbacks) {
        return new Promise((resolve, reject) => {
            const msg = new RegisterMetadataCallbacks(createRegisteredCallbacks(metadataCallbacks));
            const handler = defaultEventHandler(resolve, reject);
            this.wasmSDKWorker.postMessageAndRegisterCallbacks(msg, metadataCallbacks, handler);
        });
    }
    resetRecognizers(hardReset) {
        return new Promise((resolve, reject) => {
            const msg = new ResetRecognizers(hardReset);
            const handler = defaultEventHandler(resolve, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
    setDetectionOnlyMode(detectionOnly) {
        return new Promise((resolve, reject) => {
            const msg = new SetDetectionOnly(detectionOnly);
            const handler = defaultEventHandler(resolve, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
    setCameraPreviewMirrored(mirrored) {
        return new Promise((resolve, reject) => {
            const msg = new SetCameraPreviewMirrored(mirrored);
            const handler = defaultEventHandler(resolve, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
    delete() {
        if (this.deleted) {
            return Promise.reject("Recognizer runner is already deleted.");
        }
        return new Promise((resolve, reject) => {
            const msg = new DeleteRecognizerRunner();
            const handler = defaultEventHandler(() => {
                this.deleted = true;
                resolve();
            }, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
}
function getRecognizerHandles(remoteRecognizers) {
    const recognizerHandles = [];
    for (const remoteRecognizer of remoteRecognizers) {
        recognizerHandles.push(remoteRecognizer.getRemoteObjectHandle());
    }
    return recognizerHandles;
}
class WasmModuleWorkerProxy {
    constructor(wasmSDKWorker) {
        this.wasmSDKWorker = wasmSDKWorker;
    }
    createRecognizerRunner(recognizers, allowMultipleResults = false, metadataCallbacks = {}) {
        return new Promise((resolve, reject) => {
            const recognizerHandles = getRecognizerHandles(recognizers);
            const msg = new CreateRecognizerRunner(recognizerHandles, allowMultipleResults, createRegisteredCallbacks(metadataCallbacks));
            const handler = defaultEventHandler(() => {
                resolve(new RemoteRecognizerRunner(this.wasmSDKWorker, recognizers));
            }, reject);
            this.wasmSDKWorker.postMessageAndRegisterCallbacks(msg, metadataCallbacks, handler);
        });
    }
    /* eslint-disable @typescript-eslint/no-explicit-any */
    newRecognizer(className, ...constructorArgs) {
        return new Promise((resolve, reject) => {
            const msg = new CreateNewRecognizer(className, wrapParameters(constructorArgs));
            const handler = defaultResultEventHandler((msg) => {
                const remoteRecognizer = new RemoteRecognizer(this.wasmSDKWorker, className, msg.objectHandle);
                resolve(remoteRecognizer);
            }, reject);
            this.wasmSDKWorker.postMessage(msg, handler);
        });
    }
}
class WasmSDKWorker {
    /* eslint-enable lines-between-class-members */
    constructor(worker, loadProgressCallback, userId, rejectHandler) {
        this.eventHandlers = {};
        this.metadataCallbacks = {};
        this.loadedWasmType = exports.WasmType.Basic; // will be updated after WASM gets loaded
        this.mbWasmWorker = worker;
        this.mbWasmWorker.onmessage = (event) => { this.handleWorkerEvent(event); };
        this.mbWasmWorker.onerror = () => {
            rejectHandler("Problem during initialization of worker file!");
            return;
        };
        this.mbWasmModule = new WasmModuleWorkerProxy(this);
        this.loadCallback = loadProgressCallback;
        this.recognizersWithCallbacks = new Map();
        this.userId = userId;
        this.showOverlay = false;
    }
    postMessage(message, eventHandler) {
        this.eventHandlers[message.messageID] = eventHandler;
        this.mbWasmWorker.postMessage(message);
    }
    postTransferrableMessage(message, eventHandler) {
        this.eventHandlers[message.messageID] = eventHandler;
        this.mbWasmWorker.postMessage(message, message.getTransferrables());
    }
    postMessageAndRegisterCallbacks(message, metadataCallbacks, eventHandler) {
        this.eventHandlers[message.messageID] = eventHandler;
        this.metadataCallbacks = metadataCallbacks;
        this.mbWasmWorker.postMessage(message);
    }
    registerRecognizerCallbacks(remoteRecognizerHandle, recognizer) {
        this.recognizersWithCallbacks.set(remoteRecognizerHandle, recognizer);
    }
    unregisterRecognizerCallbacks(remoteRecognizerHandle) {
        this.recognizersWithCallbacks.delete(remoteRecognizerHandle);
    }
    /**
     * Clean up the active instance of the SDK.
     *
     * It's not possible to use the SDK after this method is called.
     */
    delete() {
        this.mbWasmWorker.terminate();
    }
    getProductIntegrationInfo() {
        return new Promise((resolve, reject) => {
            const msg = new GetProductIntegrationInfo(this.userId);
            const handler = defaultResultEventHandler((msg) => {
                resolve(msg.result);
            }, reject);
            this.postMessage(msg, handler);
        });
    }
    handleWorkerEvent(event) {
        if ("isCallbackMessage" in event.data) {
            const msg = event.data;
            switch (msg.callbackType) {
                case MetadataCallback.onDebugText:
                    if (typeof this.metadataCallbacks.onDebugText === "function") {
                        this.metadataCallbacks.onDebugText(msg.callbackParameters[0]);
                    }
                    break;
                case MetadataCallback.onDetectionFailed:
                    if (typeof this.metadataCallbacks.onDetectionFailed === "function") {
                        this.metadataCallbacks.onDetectionFailed();
                    }
                    break;
                case MetadataCallback.onPointsDetection:
                    if (typeof this.metadataCallbacks.onPointsDetection === "function") {
                        this.metadataCallbacks.onPointsDetection(msg.callbackParameters[0]);
                    }
                    break;
                case MetadataCallback.onQuadDetection:
                    if (typeof this.metadataCallbacks.onQuadDetection === "function") {
                        this.metadataCallbacks.onQuadDetection(msg.callbackParameters[0]);
                    }
                    break;
                case MetadataCallback.onFirstSideResult:
                    if (typeof this.metadataCallbacks.onFirstSideResult === "function") {
                        this.metadataCallbacks.onFirstSideResult();
                    }
                    break;
                case MetadataCallback.recognizerCallback:
                    {
                        // first parameter is address, other parameters are callback parameters
                        const address = msg.callbackParameters.shift();
                        const recognizer = this.recognizersWithCallbacks.get(address.recognizerHandle);
                        if (recognizer !== undefined) {
                            recognizer.invokeCallback(address.callbackName, msg.callbackParameters);
                        }
                        else {
                            console.warn("Cannot find recognizer to deliver callback message. Maybe it's destroyed?", address);
                        }
                        break;
                    }
                default:
                    throw new Error(`Unknown callback type: ${MetadataCallback[msg.callbackType]}`);
            }
        }
        else if ("isLoadProgressMessage" in event.data) {
            const msg = event.data;
            if (typeof this.loadCallback === "function") {
                this.loadCallback(msg.progress);
            }
        }
        else {
            const msg = event.data;
            const eventHandler = this.eventHandlers[msg.messageID];
            delete this.eventHandlers[msg.messageID];
            eventHandler(msg);
        }
    }
    static async createWasmWorker(worker, wasmLoadSettings, userId) {
        return new Promise((resolve, reject) => {
            const wasmWorker = new WasmSDKWorker(worker, wasmLoadSettings.loadProgressCallback, userId, reject);
            const initMessage = new InitMessage(wasmLoadSettings, userId);
            const initEventHandler = defaultResultEventHandler((msg) => {
                const successMsg = msg;
                wasmWorker.showOverlay = successMsg.showOverlay;
                wasmWorker.loadedWasmType = successMsg.wasmType;
                resolve(wasmWorker);
            }, 
            /* eslint-disable @typescript-eslint/no-explicit-any */
            (error) => {
                if (wasmWorker && typeof wasmWorker.delete === "function") {
                    wasmWorker.delete();
                }
                reject(error);
            }
            /* eslint-enable @typescript-eslint/no-explicit-any */
            );
            wasmWorker.postMessage(initMessage, initEventHandler);
        });
    }
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (ErrorCodes) {
    ErrorCodes["WORKER_WASM_LOAD_FAILURE"] = "WORKER_WASM_LOAD_FAILURE";
    ErrorCodes["WORKER_WASM_INIT_MISSING"] = "WORKER_WASM_INIT_MISSING";
    ErrorCodes["WORKER_FUNCTION_INVOKE_FAILURE"] = "WORKER_FUNCTION_INVOKE_FAILURE";
    ErrorCodes["WORKER_RECOGNIZER_CREATION_FAILURE"] = "WORKER_RECOGNIZER_CREATION_FAILURE";
    ErrorCodes["WORKER_RUNNER_EXISTS"] = "WORKER_RUNNER_EXISTS";
    ErrorCodes["WORKER_RUNNER_CREATION_FAILURE"] = "WORKER_RUNNER_CREATION_FAILURE";
    ErrorCodes["WORKER_RUNNER_MISSING"] = "WORKER_RUNNER_MISSING";
    ErrorCodes["WORKER_RUNNER_RECONFIGURE_FAILURE"] = "WORKER_RUNNER_RECONFIGURE_FAILURE";
    ErrorCodes["WORKER_RUNNER_DELETED"] = "WORKER_RUNNER_DELETED";
    ErrorCodes["WORKER_RUNNER_DELETE_FAILURE"] = "WORKER_RUNNER_DELETE_FAILURE";
    ErrorCodes["WORKER_OBJECT_INVOKE_FAILURE"] = "WORKER_OBJECT_INVOKE_FAILURE";
    ErrorCodes["WORKER_IMAGE_PROCESS_FAILURE"] = "WORKER_IMAGE_PROCESS_FAILURE";
    ErrorCodes["WORKER_HANDLE_UNDEFINED"] = "WORKER_HANDLE_UNDEFINED";
    ErrorCodes["WORKER_MESSAGE_ACTION_UNKNOWN"] = "WORKER_MESSAGE_ACTION_UNKNOWN";
    ErrorCodes["WORKER_LICENSE_UNLOCK_ERROR"] = "WORKER_LICENSE_UNLOCK_ERROR";
    ErrorCodes["WORKER_INTEGRATION_INFO_FAILURE"] = "WORKER_INTEGRATION_INFO_FAILURE";
    ErrorCodes["LOCAL_SDK_RUNNER_MISSING"] = "LOCAL_SDK_RUNNER_MISSING";
    ErrorCodes["LOCAL_SDK_RUNNER_EMPTY"] = "LOCAL_SDK_RUNNER_EMPTY";
    ErrorCodes["LICENSE_UNLOCK_ERROR"] = "LICENSE_UNLOCK_ERROR";
    ErrorCodes["FRAME_CAPTURE_SVG_UNSUPPORTED"] = "FRAME_CAPTURE_SVG_UNSUPPORTED";
    ErrorCodes["FRAME_CAPTURE_CANVAS_MISSING"] = "FRAME_CAPTURE_CANVAS_MISSING";
    ErrorCodes["SDK_WASM_SETTINGS_MISSING"] = "SDK_WASM_SETTINGS_MISSING";
    ErrorCodes["SDK_LICENSE_KEY_MISSING"] = "SDK_LICENSE_KEY_MISSING";
    ErrorCodes["SDK_WASM_MODULE_NAME_MISSING"] = "SDK_WASM_MODULE_NAME_MISSING";
    ErrorCodes["SDK_ENGINE_LOCATION_INVALID"] = "SDK_ENGINE_LOCATION_INVALID";
    ErrorCodes["SDK_WORKER_LOCATION_INVALID"] = "SDK_WORKER_LOCATION_INVALID";
    ErrorCodes["SDK_MISSING"] = "SDK_MISSING";
    ErrorCodes["SDK_RECOGNIZERS_MISSING"] = "SDK_RECOGNIZERS_MISSING";
    ErrorCodes["VIDEO_RECOGNIZER_ELEMENT_MISSING"] = "VIDEO_RECOGNIZER_ELEMENT_MISSING";
    ErrorCodes["VIDEO_RECOGNIZER_CAMERA_MISSING"] = "VIDEO_RECOGNIZER_CAMERA_MISSING";
    ErrorCodes["VIDEO_RECOGNIZER_CAMERA_NOT_ALLOWED"] = "VIDEO_RECOGNIZER_CAMERA_NOT_ALLOWED";
    ErrorCodes["VIDEO_RECOGNIZER_CAMERA_UNAVAILABLE"] = "VIDEO_RECOGNIZER_CAMERA_UNAVAILABLE";
    ErrorCodes["VIDEO_RECOGNIZER_CAMERA_IN_USE"] = "VIDEO_RECOGNIZER_CAMERA_IN_USE";
    ErrorCodes["VIDEO_RECOGNIZER_MEDIA_DEVICES_UNSUPPORTED"] = "VIDEO_RECOGNIZER_MEDIA_DEVICES_UNSUPPORTED";
    ErrorCodes["VIDEO_RECOGNIZER_FEED_RELEASED"] = "VIDEO_RECOGNIZER_FEED_RELEASED";
    ErrorCodes["VIDEO_RECOGNIZER_FEED_NOT_PAUSED"] = "VIDEO_RECOGNIZER_FEED_NOT_PAUSED";
    ErrorCodes["VIDEO_RECOGNIZER_PLAY_REQUEST_INTERRUPTED"] = "VIDEO_RECOGNIZER_PLAY_REQUEST_INTERRUPTED";
    ErrorCodes["VIDEO_RECOGNIZER_FEED_PAUSED"] = "VIDEO_RECOGNIZER_FEED_PAUSED";
    ErrorCodes["VIDEO_RECOGNIZER_RECOGNIZERS_RESET_FAILURE"] = "VIDEO_RECOGNIZER_RECOGNIZERS_RESET_FAILURE";
    ErrorCodes["VIDEO_RECOGNIZER_FEED_MISSING"] = "VIDEO_RECOGNIZER_FEED_MISSING";
})(exports.ErrorCodes || (exports.ErrorCodes = {}));
(function (ErrorMessages) {
    ErrorMessages["WORKER_HANDLE_UNDEFINED"] = "Cannot find object with handle: undefined";
    ErrorMessages["WORKER_WASM_LOAD_FAILURE"] = "Failed to load WASM in web worker!";
    ErrorMessages["WORKER_WASM_INIT_MISSING"] = "WASM module is not initialized!";
    ErrorMessages["WORKER_FUNCTION_INVOKE_FAILURE"] = "Failed to invoke function!";
    ErrorMessages["WORKER_RECOGNIZER_CREATION_FAILURE"] = "Failed to create new recognizer!";
    ErrorMessages["WORKER_RUNNER_EXISTS"] = "Recognizer runner is already created! Multiple instances are not allowed!";
    ErrorMessages["WORKER_RUNNER_CREATION_FAILURE"] = "Failed to create new recognizer runner!";
    ErrorMessages["WORKER_RUNNER_MISSING"] = "Recognizer runner is not created! There is nothing to reconfigure!";
    ErrorMessages["WORKER_RUNNER_RECONFIGURE_FAILURE"] = "Failed to reconfigure recognizer runner!";
    ErrorMessages["WORKER_RUNNER_DELETED"] = "Recognizer runner is already deleted!";
    ErrorMessages["WORKER_RUNNER_DELETE_FAILURE"] = "Failed to delete recognizer runner!";
    ErrorMessages["WORKER_OBJECT_INVOKE_FAILURE"] = "Failed to invoke object!";
    ErrorMessages["WORKER_IMAGE_PROCESS_FAILURE"] = "Recognizer runner is not initialized! Cannot process image!";
    ErrorMessages["WORKER_INTEGRATION_INFO_FAILURE"] = "Failed to get product integration info!";
    ErrorMessages["LOCAL_SDK_RUNNER_MISSING"] = "Property nativeRecognizerRunner is not available!";
    ErrorMessages["LOCAL_SDK_RUNNER_EMPTY"] = "Native RecognizerRunner cannot be empty!";
    ErrorMessages["LICENSE_TOKEN_STATE_INCORRECT"] = "Internal error (Incorrect token state)";
    ErrorMessages["LICENSE_PAYLOAD_VERIFICATION_FAILED"] = "Failed to verify server permission's digital signature!";
    ErrorMessages["LICENSE_PAYLOAD_CORRUPTED"] = "Server permission payload is corrupted!";
    ErrorMessages["LICENSE_PERMISSION_EXPIRED"] = "Internal error (server permission expired)";
    ErrorMessages["LICENSE_REMOTE_LOCKED"] = "Provided license key has been remotely locked. Please contact support for more information!";
    ErrorMessages["FRAME_CAPTURE_SVG_UNSUPPORTED"] = "Recognition of SVG elements not supported!";
    ErrorMessages["FRAME_CAPTURE_CANVAS_MISSING"] = "Could not get canvas 2d context!";
    ErrorMessages["SDK_WASM_SETTINGS_MISSING"] = "Missing WASM load settings!";
    ErrorMessages["SDK_LICENSE_KEY_MISSING"] = "Missing license key!";
    ErrorMessages["SDK_WASM_MODULE_NAME_MISSING"] = "Missing WASM module name!";
    ErrorMessages["SDK_ENGINE_LOCATION_INVALID"] = "Setting property 'engineLocation' must be a string!";
    ErrorMessages["SDK_WORKER_LOCATION_INVALID"] = "Setting property 'workerLocation' must be a string!";
    ErrorMessages["SDK_MISSING"] = "SDK is not provided!";
    ErrorMessages["SDK_RECOGNIZERS_MISSING"] = "To create RecognizerRunner at least 1 recognizer is required.";
    ErrorMessages["VIDEO_RECOGNIZER_ELEMENT_MISSING"] = "Video element, i.e. camera feed is not provided!";
    ErrorMessages["VIDEO_RECOGNIZER_CAMERA_MISSING"] = "Camera not found!";
    ErrorMessages["VIDEO_RECOGNIZER_CAMERA_NOT_ALLOWED"] = "Camera not allowed!";
    ErrorMessages["VIDEO_RECOGNIZER_CAMERA_UNAVAILABLE"] = "Camera not available!";
    ErrorMessages["VIDEO_RECOGNIZER_CAMERA_IN_USE"] = "Camera in use!";
    ErrorMessages["VIDEO_RECOGNIZER_MEDIA_DEVICES_UNSUPPORTED"] = "Media devices not supported by browser.";
    ErrorMessages["VIDEO_RECOGNIZER_FEED_RELEASED"] = "The associated video feed has been released!";
    ErrorMessages["VIDEO_RECOGNIZER_FEED_NOT_PAUSED"] = "The associated video feed is not paused. Use resumeRecognition instead!";
    ErrorMessages["VIDEO_RECOGNIZER_PLAY_REQUEST_INTERRUPTED"] = "The play() request was interrupted or prevented by browser security rules!";
    ErrorMessages["VIDEO_RECOGNIZER_FEED_PAUSED"] = "Cannot resume recognition while video feed is paused! Use recognize or startRecognition";
    ErrorMessages["VIDEO_RECOGNIZER_RECOGNIZERS_RESET_FAILURE"] = "Could not reset recognizers!";
    ErrorMessages["VIDEO_RECOGNIZER_FEED_MISSING"] = "Missing video feed!";
})(exports.ErrorMessages || (exports.ErrorMessages = {}));
const videoRecognizerErrors = {
    feedMissing: {
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_FEED_MISSING,
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_FEED_MISSING,
    },
    recognizersResetFailure: {
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_RECOGNIZERS_RESET_FAILURE,
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_RECOGNIZERS_RESET_FAILURE,
    },
    feedPaused: {
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_FEED_PAUSED,
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_FEED_PAUSED,
    },
    playRequestInterrupted: {
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_PLAY_REQUEST_INTERRUPTED,
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_PLAY_REQUEST_INTERRUPTED,
    },
    videoFeedNotPaused: {
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_FEED_NOT_PAUSED,
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_FEED_NOT_PAUSED,
    },
    videoFeedReleased: {
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_FEED_RELEASED,
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_FEED_RELEASED,
    },
    mediaDevicesUnsupported: {
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_MEDIA_DEVICES_UNSUPPORTED,
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_MEDIA_DEVICES_UNSUPPORTED,
    },
    cameraMissing: {
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_CAMERA_MISSING,
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_CAMERA_MISSING,
    },
    cameraNotAllowed: {
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_CAMERA_NOT_ALLOWED,
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_CAMERA_NOT_ALLOWED,
    },
    elementMissing: {
        message: exports.ErrorMessages.VIDEO_RECOGNIZER_ELEMENT_MISSING,
        code: exports.ErrorCodes.VIDEO_RECOGNIZER_ELEMENT_MISSING,
    },
};
const sdkErrors = {
    wasmSettingsMissing: {
        message: exports.ErrorMessages.SDK_WASM_SETTINGS_MISSING,
        code: exports.ErrorCodes.SDK_WASM_SETTINGS_MISSING,
    },
    licenseKeyMissing: {
        message: exports.ErrorMessages.SDK_LICENSE_KEY_MISSING,
        code: exports.ErrorCodes.SDK_LICENSE_KEY_MISSING,
    },
    wasmModuleNameMissing: {
        message: exports.ErrorMessages.SDK_WASM_MODULE_NAME_MISSING,
        code: exports.ErrorCodes.SDK_WASM_MODULE_NAME_MISSING,
    },
    engineLocationInvalid: {
        message: exports.ErrorMessages.SDK_ENGINE_LOCATION_INVALID,
        code: exports.ErrorCodes.SDK_ENGINE_LOCATION_INVALID,
    },
    workerLocationInvalid: {
        message: exports.ErrorMessages.SDK_WORKER_LOCATION_INVALID,
        code: exports.ErrorCodes.SDK_WORKER_LOCATION_INVALID,
    },
    missing: {
        message: exports.ErrorMessages.SDK_MISSING,
        code: exports.ErrorCodes.SDK_MISSING,
    },
    recognizersMissing: {
        message: exports.ErrorMessages.SDK_RECOGNIZERS_MISSING,
        code: exports.ErrorCodes.SDK_RECOGNIZERS_MISSING,
    },
};
const frameCaptureErrors = {
    svgUnsupported: {
        message: exports.ErrorMessages.FRAME_CAPTURE_SVG_UNSUPPORTED,
        code: exports.ErrorCodes.FRAME_CAPTURE_SVG_UNSUPPORTED,
    },
    canvasMissing: {
        message: exports.ErrorMessages.FRAME_CAPTURE_CANVAS_MISSING,
        code: exports.ErrorCodes.FRAME_CAPTURE_CANVAS_MISSING,
    },
};
const licenseErrors = {
    licenseTokenStateIncorrect: {
        code: exports.ErrorCodes.LICENSE_UNLOCK_ERROR,
        message: exports.ErrorMessages.LICENSE_TOKEN_STATE_INCORRECT,
    },
    licensePayloadVerificationFailed: {
        code: exports.ErrorCodes.LICENSE_UNLOCK_ERROR,
        message: exports.ErrorMessages.LICENSE_PAYLOAD_VERIFICATION_FAILED,
    },
    licensePayloadCorrupted: {
        code: exports.ErrorCodes.LICENSE_UNLOCK_ERROR,
        message: exports.ErrorMessages.LICENSE_PAYLOAD_CORRUPTED,
    },
    licensePermissionExpired: {
        code: exports.ErrorCodes.LICENSE_UNLOCK_ERROR,
        message: exports.ErrorMessages.LICENSE_PERMISSION_EXPIRED,
    },
    licenseRemoteLocked: {
        code: exports.ErrorCodes.LICENSE_UNLOCK_ERROR,
        message: exports.ErrorMessages.LICENSE_REMOTE_LOCKED,
    },
    licenseNetworkError: {
        code: exports.ErrorCodes.LICENSE_UNLOCK_ERROR,
    },
    licenseInvalid: {
        code: exports.ErrorCodes.LICENSE_UNLOCK_ERROR,
    },
};
const localSdkErrors = {
    runnerMissing: {
        message: exports.ErrorMessages.LOCAL_SDK_RUNNER_MISSING,
        code: exports.ErrorCodes.LOCAL_SDK_RUNNER_MISSING,
    },
    runnerEmpty: {
        message: exports.ErrorMessages.LOCAL_SDK_RUNNER_EMPTY,
        code: exports.ErrorCodes.LOCAL_SDK_RUNNER_EMPTY,
    },
};
const workerErrors = {
    imageProcessFailure: {
        message: exports.ErrorMessages.WORKER_IMAGE_PROCESS_FAILURE,
        code: exports.ErrorCodes.WORKER_IMAGE_PROCESS_FAILURE,
    },
    objectInvokeFailure: {
        message: exports.ErrorMessages.WORKER_OBJECT_INVOKE_FAILURE,
        code: exports.ErrorCodes.WORKER_OBJECT_INVOKE_FAILURE,
    },
    runnerDeleteFailure: {
        message: exports.ErrorMessages.WORKER_RUNNER_DELETE_FAILURE,
        code: exports.ErrorCodes.WORKER_RUNNER_DELETE_FAILURE,
    },
    runnerDeleted: {
        message: exports.ErrorMessages.WORKER_RUNNER_DELETED,
        code: exports.ErrorCodes.WORKER_RUNNER_DELETED,
    },
    runnerReconfigureFailure: {
        message: exports.ErrorMessages.WORKER_RUNNER_RECONFIGURE_FAILURE,
        code: exports.ErrorCodes.WORKER_RUNNER_RECONFIGURE_FAILURE,
    },
    runnerMissing: {
        message: exports.ErrorMessages.WORKER_RUNNER_MISSING,
        code: exports.ErrorCodes.WORKER_RUNNER_MISSING,
    },
    runnerCreationFailure: {
        message: exports.ErrorMessages.WORKER_RUNNER_CREATION_FAILURE,
        code: exports.ErrorCodes.WORKER_RUNNER_CREATION_FAILURE,
    },
    runnerExists: {
        message: exports.ErrorMessages.WORKER_RUNNER_EXISTS,
        code: exports.ErrorCodes.WORKER_RUNNER_EXISTS,
    },
    recognizerCreationFailure: {
        message: exports.ErrorMessages.WORKER_RECOGNIZER_CREATION_FAILURE,
        code: exports.ErrorCodes.WORKER_RECOGNIZER_CREATION_FAILURE,
    },
    functionInvokeFailure: {
        message: exports.ErrorMessages.WORKER_FUNCTION_INVOKE_FAILURE,
        code: exports.ErrorCodes.WORKER_FUNCTION_INVOKE_FAILURE,
    },
    wasmInitMissing: {
        message: exports.ErrorMessages.WORKER_WASM_INIT_MISSING,
        code: exports.ErrorCodes.WORKER_WASM_INIT_MISSING,
    },
    wasmLoadFailure: {
        message: exports.ErrorMessages.WORKER_WASM_LOAD_FAILURE,
        code: exports.ErrorCodes.WORKER_WASM_LOAD_FAILURE,
    },
    handleUndefined: {
        message: exports.ErrorMessages.WORKER_HANDLE_UNDEFINED,
        code: exports.ErrorCodes.WORKER_HANDLE_UNDEFINED,
    },
    integrationInfoFailure: {
        message: exports.ErrorMessages.WORKER_INTEGRATION_INFO_FAILURE,
        code: exports.ErrorCodes.WORKER_INTEGRATION_INFO_FAILURE,
    },
};

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (PreferredCameraType) {
    /** Prefer back facing camera */
    PreferredCameraType[PreferredCameraType["BackFacingCamera"] = 0] = "BackFacingCamera";
    /** Prefer front facing camera */
    PreferredCameraType[PreferredCameraType["FrontFacingCamera"] = 1] = "FrontFacingCamera";
})(exports.PreferredCameraType || (exports.PreferredCameraType = {}));
// inspired by https://unpkg.com/browse/scandit-sdk@4.6.1/src/lib/cameraAccess.ts
const backCameraKeywords = [
    "rear",
    "back",
    "rück",
    "arrière",
    "trasera",
    "trás",
    "traseira",
    "posteriore",
    "posterior",
    "后面",
    "後面",
    "背面",
    "后置",
    "後置",
    "背置",
    "задней",
    "الخلفية",
    "후",
    "arka",
    "achterzijde",
    "หลัง",
    "baksidan",
    "bagside",
    "sau",
    "bak",
    "tylny",
    "takakamera",
    "belakang",
    "אחורית",
    "πίσω",
    "spate",
    "hátsó",
    "zadní",
    "darrere",
    "zadná",
    "задня",
    "stražnja",
    "belakang",
    "बैक"
];
const backDualWideCameraLocalizations = [
    "Cameră dublă cu obiectiv superangular spate",
    "מצלמה כפולה רחבה אחורית",
    "Артқы қос кең бұрышты камера",
    "Câmara grande angular dupla traseira",
    "Πίσω διπλή ευρεία κάμερα",
    "後置雙廣角鏡頭相機",
    "Задна двойна широкоъгълна камера",
    "Càmera dual posterior amb gran angular",
    "Zadná duálna širokouhlá kamera",
    "كاميرا خلفية مزدوجة عريضة",
    "Задняя двойная широкоугольная камера",
    "Задня здвоєна ширококутна камера",
    "Cámara amplia posterior doble",
    "Dwikamera Lebar Belakang",
    "Tylny dwuobiektywowy aparat szerokokątny",
    "Dubbel vidvinkelkamera på baksidan",
    "Back Dual Wide Camera",
    "Hátsó, kettős, széles látószögű kamera",
    "후면 듀얼 와이드 카메라",
    "Double caméra grand angle arrière",
    "Fotocamera doppia con grandangolo (posteriore)",
    "Double appareil photo grand angle arrière",
    "Zadní duální širokoúhlý fotoaparát",
    "Çift Geniş Kamera Arka Yüzü",
    "Laajakulmainen kaksoistakakamera",
    "Rückseitige Dual-Weitwinkelkamera",
    "बैक ड्युअल वाइड कैमरा",
    "后置双广角镜头",
    "Câmera Dupla Grande-Angular Traseira",
    "後置雙廣角相機",
    "กล้องคู่ด้านหลังมุมกว้าง",
    "Kamera Lebar Belakang Ganda",
    "Dobbelt vidvinkelkamera bak",
    "Camera kép rộng mặt sau",
    "Cámara trasera dual con gran angular",
    "背面デュアル広角カメラ",
    "Stražnja dvostruka široka kamera"
];
const backCameraLocalizations = [
    "후면 카메라",
    "後置相機",
    "Задна камера",
    "後置鏡頭",
    "Camera mặt sau",
    "Hátoldali kamera",
    "Cámara trasera",
    "Back Camera",
    "Kamera på baksidan",
    "Πίσω κάμερα",
    "Bagsidekamera",
    "Zadná kamera",
    "Fotocamera (posteriore)",
    "Câmara traseira",
    "מצלמה אחורית",
    "Takakamera",
    "Rückkamera",
    "Caméra arrière",
    "Zadní fotoaparát",
    "Артқы камера",
    "Tylny aparat",
    "बैक कैमरा",
    "Hátsó kamera",
    "Camera aan achterzijde",
    "Kamera Belakang",
    "Câmera Traseira",
    "Stražnja kamera",
    "الكاميرا الخلفية",
    "Càmera posterior",
    "Fotocamera posteriore",
    "Càmera del darrere",
    "กล้องด้านหลัง",
    "Cameră spate",
    "Kamera, bagside",
    "背面カメラ",
    "Задня камера",
    "Arka Kamera",
    "后置相机",
    "Камера на задней панели",
    "后置镜头",
    "Kamera bak",
    "Задняя камера",
    "Aparat tylny",
    "Kamera på baksiden",
    "Câmera de Trás"
];
const isAndroidDevice = () => {
    const u = navigator.userAgent;
    return !!u.match(/Android/i);
};
const isIOSDevice = () => {
    const u = navigator.userAgent;
    return !!u.match(/iPhone/i);
};
const isMobileDevice = () => {
    return isAndroidDevice() || isIOSDevice();
};
function isBackCameraLabel(label) {
    const lowercaseLabel = label.toLowerCase();
    return backCameraKeywords.some(keyword => lowercaseLabel.includes(keyword));
}
class SelectedCamera {
    constructor(mdi, facing, label) {
        this.deviceId = mdi.deviceId;
        this.facing = facing;
        this.groupId = mdi.groupId;
        // apply custom label
        if (label) {
            this.label = label;
        }
        else {
            this.label = mdi.label;
        }
    }
}
async function getCameraDevices() {
    const frontCameras = [];
    const backCameras = [];
    let devices = await navigator.mediaDevices.enumerateDevices();
    // if permission is not given, label of video devices will be empty string
    if (devices.filter(device => device.kind === "videoinput").every(device => device.label === "")) {
        const stream = await navigator.mediaDevices.getUserMedia({
            video: {
                facingMode: { ideal: "environment" }
            },
            audio: false
        });
        // enumerate devices again - now the label field should be non-empty, as we have a stream active
        // (even if we didn't get persistent permission for camera)
        devices = await navigator.mediaDevices.enumerateDevices();
        // close the stream, as we don't need it anymore
        stream.getTracks().forEach(track => track.stop());
    }
    const cameras = devices.filter(device => device.kind === "videoinput");
    let backCameraIterator = 0;
    let frontCameraIterator = 0;
    for (const camera of cameras) {
        // phone back camera
        if (isBackCameraLabel(camera.label)) {
            backCameraIterator++;
            let backLabel = undefined;
            // we apply a custom label on Android devices
            if (isAndroidDevice()) {
                backLabel = `Back camera ${backCameraIterator}`;
            }
            backCameras.push(new SelectedCamera(camera, exports.PreferredCameraType.BackFacingCamera, backLabel));
        }
        else 
        // front camera or non-phone camera
        {
            frontCameraIterator++;
            let frontLabel = undefined;
            if (isAndroidDevice()) {
                // we apply a custom label on Android devices
                frontLabel = `Front camera ${frontCameraIterator}`;
            }
            frontCameras.push(new SelectedCamera(camera, exports.PreferredCameraType.FrontFacingCamera, frontLabel));
        }
    }
    return {
        frontCameras,
        backCameras
    };
}
async function selectCamera(cameraId, preferredCameraType) {
    const { frontCameras, backCameras } = await getCameraDevices();
    if (!frontCameras.length && !backCameras.length) {
        return null;
    }
    // Picks camera based on the provided device id, if user provided device id up front
    if (cameraId) {
        let cameraDevice;
        cameraDevice = frontCameras.find(device => device.deviceId === cameraId);
        if (!cameraDevice) {
            cameraDevice = backCameras.find(device => device.deviceId === cameraId);
        }
        return cameraDevice || null;
    }
    let cameraDevice = null;
    if (isIOSDevice() && preferredCameraType === exports.PreferredCameraType.BackFacingCamera) {
        // If device is an iOS and preferred camera is back facing
        // pick camera which matches the localized 'Back Dual Wide Camera'
        let selectedCamera = backCameras.find(camera => backDualWideCameraLocalizations.includes(camera.label));
        if (!selectedCamera) {
            selectedCamera = backCameras.find(camera => backCameraLocalizations.includes(camera.label));
        }
        if (selectedCamera) {
            cameraDevice = selectedCamera;
        }
    }
    else if (isAndroidDevice() && preferredCameraType === exports.PreferredCameraType.BackFacingCamera) {
        let bestCameraDevice = {
            deviceId: "",
            score: -1,
        };
        const calculateScore = (hasTorch, hasSingleShot) => {
            let score = 0;
            if (hasTorch)
                score++;
            if (hasSingleShot)
                score++;
            return score;
        };
        for (const backFacingCamera of backCameras) {
            const mediaStream = await navigator.mediaDevices.getUserMedia({
                video: {
                    deviceId: backFacingCamera.deviceId,
                    width: 1920,
                    height: 1080,
                },
            });
            if ("getCapabilities" in mediaStream.getVideoTracks()[0]) {
                const capabilities = mediaStream.getVideoTracks()[0].getCapabilities();
                // @ts-expect-error Property will exist on object
                const hasTorch = Boolean(capabilities.torch);
                // @ts-expect-error Property will exist on object
                const hasSingleShot = capabilities.focusMode?.includes("single-shot");
                const cameraScore = calculateScore(hasTorch, hasSingleShot);
                if (cameraScore > bestCameraDevice.score) {
                    bestCameraDevice = {
                        deviceId: backFacingCamera.deviceId,
                        score: cameraScore,
                    };
                }
            }
            closeStreamTracks(mediaStream);
        }
        cameraDevice = backCameras.find(camera => camera.deviceId === bestCameraDevice.deviceId) || null;
    }
    if (cameraDevice === null) {
        // Case where camera device is still not found, so we revert back to "old" logic
        // Decide from which array the camera will be selected
        let cameraPool = (backCameras.length > 0 ? backCameras : frontCameras);
        // If there is at least one back facing camera and user prefers back facing camera, use that as a selection pool
        if (preferredCameraType === exports.PreferredCameraType.BackFacingCamera && backCameras.length > 0) {
            cameraPool = backCameras;
        }
        // If there is at least one front facing camera and is preferred by user, use that as a selection pool
        if (preferredCameraType === exports.PreferredCameraType.FrontFacingCamera && frontCameras.length > 0) {
            cameraPool = frontCameras;
        }
        // Sort camera pool by label
        cameraPool = cameraPool.sort((camera1, camera2) => camera1.label.localeCompare(camera2.label));
        // Check if cameras are labeled with resolution information, take the higher-resolution one in that case
        // Otherwise pick the last camera (Samsung wide on most Android devices)
        let selectedCameraIndex = cameraPool.length - 1;
        // Gets camera resolutions from the device name, if exists
        const cameraResolutions = cameraPool.map(camera => {
            const regExp = RegExp(/\b([0-9]+)MP?\b/, "i");
            const match = regExp.exec(camera.label);
            if (match !== null) {
                return parseInt(match[1], 10);
            }
            else {
                return NaN;
            }
        });
        // Picks camera  based on highest resolution in the name
        if (!cameraResolutions.some(cameraResolution => isNaN(cameraResolution))) {
            selectedCameraIndex = cameraResolutions.lastIndexOf(Math.max(...cameraResolutions));
        }
        return cameraPool[selectedCameraIndex];
    }
    return cameraDevice;
}
const closeStreamTracks = (stream) => {
    const tracks = stream.getTracks();
    for (const track of tracks) {
        track.stop();
    }
};
/**
 * Bind camera device to video feed (HTMLVideoElement).
 *
 * This function will return `true` in case that video feed of camera device has been flipped,
 * and `false` otherwise.
 *
 * @param camera                Camera device which should be binded with the video element.
 * @param videoFeed             HTMLVideoElement to which camera device should be binded.
 * @param preferredCameraType   Enum representing whether to use front facing or back facing camera.
 */
async function bindCameraToVideoFeed(camera, videoFeed, preferredCameraType = exports.PreferredCameraType.BackFacingCamera) {
    const constraints = {
        audio: false,
        video: {
            width: {
                min: 640,
                ideal: 1920,
                max: 1920,
            },
            height: {
                min: 480,
                ideal: 1080,
                max: 1080
            }
        }
    };
    if (camera.deviceId === "") {
        const isPreferredBackFacing = preferredCameraType === exports.PreferredCameraType.BackFacingCamera;
        constraints.video.facingMode =
            {
                ideal: isPreferredBackFacing ? "environment" : "user"
            };
    }
    else {
        constraints.video.deviceId =
            {
                exact: camera.deviceId
            };
    }
    const stream = await navigator.mediaDevices.getUserMedia(constraints);
    videoFeed.controls = false;
    videoFeed.srcObject = stream;
    let cameraFlipped = false;
    if (camera.facing === exports.PreferredCameraType.FrontFacingCamera) {
        cameraFlipped = true;
    }
    return cameraFlipped;
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (ImageOrientation) {
    /**
     * Image contents are rotated 90 degrees left.
     * This usually happens on mobile devices when capturing image while
     * device is held in "portrait" orientation, while device camera sensor
     * is mounted horizontally (i.e. produced image is in "landscape" orienation).
     */
    ImageOrientation[ImageOrientation["RotatedLeft90"] = 0] = "RotatedLeft90";
    /**
     * Image contents are not rotated in any manner.
     * This is the default for images captured using HTML canvas, as
     * used in FrameCapture class.
     * This orientation also usually happens on mobile devices when capturing
     * image while device is held in "landscape" orientation, while device
     * camera sensor is mounted horizontally (i.e. also in same orientation).
     */
    ImageOrientation[ImageOrientation["NoRotation"] = 1] = "NoRotation";
    /**
     * Image contents are rotated 90 degrees right.
     * This usually happens on mobile devices when capturing image while
     * device is held in "reverse-portrait" orientation, while device camera sensor
     * is mounted horizontally (i.e. produced image is in "landscape" orienation).
     */
    ImageOrientation[ImageOrientation["RotatedRight90"] = 2] = "RotatedRight90";
    /**
     * Image contents are rotated 180 degrees, i.e. image contents are "upside down".
     * This usually happens on mobile devices when capturing image while
     * device is held in "reverse-landscape" orientation, while device camera sensor
     * is mounted horizontally (i.e. produced image is in "landscape" orienation).
     */
    ImageOrientation[ImageOrientation["Rotated180"] = 3] = "Rotated180";
})(exports.ImageOrientation || (exports.ImageOrientation = {}));
(function (RecognizerResultState) {
    /** Nothing has been recognized. */
    RecognizerResultState[RecognizerResultState["Empty"] = 0] = "Empty";
    /** Something has been recognized, but some mandatory data is still missing. */
    RecognizerResultState[RecognizerResultState["Uncertain"] = 1] = "Uncertain";
    /** All required data has been recognized. */
    RecognizerResultState[RecognizerResultState["Valid"] = 2] = "Valid";
    /** Single stage of a multi-stage recognition is finished. */
    RecognizerResultState[RecognizerResultState["StageValid"] = 3] = "StageValid";
})(exports.RecognizerResultState || (exports.RecognizerResultState = {}));

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
/**
 * Checks if browser is supported by the SDK. The minimum requirements for the browser is
 * the support for WebAssembly. If your browser does not support executing WebAssembly,
 * this function will return `false`.
 */
function isBrowserSupported() {
    // based on https://stackoverflow.com/a/47880734
    try {
        if (typeof WebAssembly === "object" && typeof WebAssembly.instantiate === "function") {
            const module = new WebAssembly.Module(Uint8Array.of(0x0, 0x61, 0x73, 0x6d, 0x01, 0x00, 0x00, 0x00));
            if (module instanceof WebAssembly.Module)
                return new WebAssembly.Instance(module) instanceof WebAssembly.Instance;
        }
    }
    catch (ignored) {
        return false;
    }
    return false;
}
/**
 * Check if current browser is in-app / embedded.
 * Detects Instagram, Facebook, LinkedIn, Twitter, WeChat, Whatsapp, and Tiktok.
 * @returns Boolean whether the browser is in-app or not
 */
function isInAppBrowser() {
    const inAppRegex = /(instagram|fbav|linkedinapp|twitter|micromessenger|whatsapp|tiktok)[/\s]?([\w.]*)/i;
    const userAgent = navigator.userAgent || navigator.vendor;
    return !!inAppRegex.exec(userAgent);
}
/**
 * Check if browser supports ES6, which is prerequisite for this SDK to execute.
 *
 * IMPORTANT: it's not possible to run this function from MicroblinkSDK if browser doesn't support
 * ES6 since this file won't be able to load.
 *
 * This function is here as a placeholder so it can be copied to standalone JS file or directly into 'index.html'.
 */
// export function isES6Supported(): boolean
// {
//     if ( typeof Symbol === "undefined" )
//     {
//         return false;
//     }
//     try
//     {
//         eval( "class Foo {}" );
//         eval( "var bar = (x) => x+1" );
//     }
//     catch ( e )
//     {
//         return false;
//     }
//     return true;
// }

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (DocumentSide) {
    DocumentSide[DocumentSide["Front"] = 0] = "Front";
    DocumentSide[DocumentSide["Back"] = 1] = "Back";
})(exports.DocumentSide || (exports.DocumentSide = {}));

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
// ============================================ /
// Frame capture and camera management support. /
// ============================================ /
let canvas;
let ctx;
/**
 * Represents a captured frame from HTMLVideoElement.
 */
class CapturedFrame {
    constructor(imageData, orientation, videoFrame) {
        // workaround for memory leak: https://github.com/ivancuric/memory-leak-repro
        const fakeImageData = {
            data: imageData.data,
            width: imageData.width,
            height: imageData.height,
            colorSpace: imageData.colorSpace,
        };
        this.imageData = fakeImageData;
        this.orientation = orientation;
        this.videoFrame = videoFrame;
    }
}
/**
 * Captures a frame from any CanvasImageSource, such as HTMLVideoElement or HTMLImageElement.
 * @param imageSource image source from which frame should be captured
 * @returns instance of CapturedFrame
 */
function captureFrame(imageSource, shouldCrop = false) {
    let imageWidth;
    let imageHeight;
    let videoFrame = false;
    if (imageSource instanceof HTMLVideoElement) {
        imageWidth = imageSource.videoWidth;
        imageHeight = imageSource.videoHeight;
        videoFrame = true;
    }
    else if (imageSource instanceof HTMLImageElement) {
        imageWidth = imageSource.naturalWidth;
        imageHeight = imageSource.naturalHeight;
    }
    else if (imageSource instanceof SVGImageElement) {
        throw new SDKError(frameCaptureErrors.svgUnsupported);
    }
    else {
        imageWidth = imageSource.width;
        imageHeight = imageSource.height;
    }
    canvas = canvas || document.createElement("canvas");
    if (canvas.width !== imageWidth && canvas.height !== imageHeight) {
        canvas.width = imageWidth;
        canvas.height = imageHeight;
    }
    ctx = ctx || canvas.getContext("2d", { willReadFrequently: true,
        alpha: false });
    if (!ctx) {
        throw new SDKError(frameCaptureErrors.canvasMissing);
    }
    ctx.drawImage(imageSource, 0, 0, canvas.width, canvas.height);
    /**
     * Take full image or cropped image based on focus problems.
     *
     * iPhone 14 Pro has focus problems, i.e. user should place a document somehow
     * far away from the camera to get the proper focus.
     *
     * In that case we're getting much bigger images (4K), and we're cropping the center
     * of the image which is then sent to processing - rectangle that is 66% size of the
     * original image is cropped from the image center.
     */
    const cropFactor = shouldCrop ? 0.66 : 1;
    const targetWidth = canvas.width * cropFactor;
    const targetHeight = canvas.height * cropFactor;
    const targetX = (canvas.width - targetWidth) / 2;
    const targetY = (canvas.height - targetHeight) / 2;
    const pixelData = ctx.getImageData(targetX, targetY, targetWidth, targetHeight);
    return new CapturedFrame(pixelData, 
    // TODO: https://developer.mozilla.org/en-US/docs/Web/API/Screen/orientation
    // or https://developer.mozilla.org/en-US/docs/Web/API/Window/orientation
    exports.ImageOrientation.NoRotation, videoFrame);
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (LicenseTokenState) {
    LicenseTokenState[LicenseTokenState["Invalid"] = 0] = "Invalid";
    LicenseTokenState[LicenseTokenState["RequiresServerPermission"] = 1] = "RequiresServerPermission";
    LicenseTokenState[LicenseTokenState["Valid"] = 2] = "Valid";
})(exports.LicenseTokenState || (exports.LicenseTokenState = {}));
(function (LicenseErrorType) {
    LicenseErrorType["LicenseTokenStateInvalid"] = "LICENSE_TOKEN_STATE_INVALID";
    LicenseErrorType["NetworkError"] = "NETWORK_ERROR";
    LicenseErrorType["RemoteLock"] = "REMOTE_LOCK";
    LicenseErrorType["PermissionExpired"] = "PERMISSION_EXPIRED";
    LicenseErrorType["PayloadCorrupted"] = "PAYLOAD_CORRUPTED";
    LicenseErrorType["PayloadSignatureVerificationFailed"] = "PAYLOAD_SIGNATURE_VERIFICATION_FAILED";
    LicenseErrorType["IncorrectTokenState"] = "INCORRECT_TOKEN_STATE";
})(exports.LicenseErrorType || (exports.LicenseErrorType = {}));
const baltazar = "https://baltazar.microblink.com/api/v2/status/check";
function toBaltazarRequest(unlockResult) {
    return {
        licenseId: unlockResult.licenseId,
        licensee: unlockResult.licensee,
        applicationIds: unlockResult.applicationIds,
        packageName: unlockResult.packageName,
        platform: "Browser",
        sdkName: unlockResult.sdkName,
        sdkVersion: unlockResult.sdkVersion
    };
}
function shouldShowOverlay(isTrial, allowRemoveDemoOverlay, allowRemoveProductionOverlay) {
    if (isTrial && allowRemoveDemoOverlay) {
        return false;
    }
    if (!isTrial && allowRemoveProductionOverlay) {
        return false;
    }
    return true;
}
(function (ServerPermissionSubmitResultStatus) {
    ServerPermissionSubmitResultStatus[ServerPermissionSubmitResultStatus["Ok"] = 0] = "Ok";
    ServerPermissionSubmitResultStatus[ServerPermissionSubmitResultStatus["NetworkError"] = 1] = "NetworkError";
    ServerPermissionSubmitResultStatus[ServerPermissionSubmitResultStatus["RemoteLock"] = 2] = "RemoteLock";
    ServerPermissionSubmitResultStatus[ServerPermissionSubmitResultStatus["PermissionExpired"] = 3] = "PermissionExpired";
    ServerPermissionSubmitResultStatus[ServerPermissionSubmitResultStatus["PayloadCorrupted"] = 4] = "PayloadCorrupted";
    ServerPermissionSubmitResultStatus[ServerPermissionSubmitResultStatus["PayloadSignatureVerificationFailed"] = 5] = "PayloadSignatureVerificationFailed";
    ServerPermissionSubmitResultStatus[ServerPermissionSubmitResultStatus["IncorrectTokenState"] = 6] = "IncorrectTokenState";
})(exports.ServerPermissionSubmitResultStatus || (exports.ServerPermissionSubmitResultStatus = {}));
/* eslint-disable @typescript-eslint/no-explicit-any,
                  @typescript-eslint/explicit-module-boundary-types,
                  @typescript-eslint/no-unsafe-member-access,
                  @typescript-eslint/no-unsafe-call
*/
async function obtainNewServerPermission(unlockResult, wasmModule) {
    // request permission from Baltazar service
    try {
        const response = await fetch(baltazar, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            cache: "no-cache",
            body: JSON.stringify(toBaltazarRequest(unlockResult))
        });
        if (response.ok) {
            const serverPermission = (await response.text()).toString();
            const result = wasmModule.submitServerPermission(serverPermission);
            return result;
        }
        else {
            return {
                status: exports.ServerPermissionSubmitResultStatus.NetworkError,
                lease: 0,
                networkErrorDescription: `Server responded with status ${response.status}`
            };
        }
    }
    catch (error) {
        return {
            status: exports.ServerPermissionSubmitResultStatus.NetworkError,
            lease: 0,
            networkErrorDescription: `Unexpected error: ${JSON.stringify(error)}`
        };
    }
}
async function unlockWasmSDK(licenseKey, allowHelloMessage, userId, wasmModule) {
    const unlockResult = wasmModule.initializeWithLicenseKey(licenseKey, userId, allowHelloMessage);
    switch (unlockResult.unlockResult) {
        case exports.LicenseTokenState.Invalid:
            return {
                error: new SDKError({
                    ...licenseErrors.licenseInvalid,
                    message: unlockResult.licenseError
                }, {
                    type: exports.LicenseErrorType.LicenseTokenStateInvalid,
                }),
            };
        case exports.LicenseTokenState.Valid:
            return {
                error: null,
                showOverlay: shouldShowOverlay(unlockResult.isTrial, unlockResult.allowRemoveDemoOverlay, unlockResult.allowRemoveProductionOverlay)
            };
        case exports.LicenseTokenState.RequiresServerPermission:
            {
                const serverPermission = await obtainNewServerPermission(unlockResult, wasmModule);
                switch (serverPermission.status) {
                    case exports.ServerPermissionSubmitResultStatus.Ok:
                        return {
                            error: null,
                            lease: serverPermission.lease
                        };
                    case exports.ServerPermissionSubmitResultStatus.NetworkError:
                        {
                            let additionalInfo = "";
                            if (serverPermission.networkErrorDescription) {
                                additionalInfo = " " + serverPermission.networkErrorDescription;
                            }
                            return {
                                error: new SDKError({
                                    ...licenseErrors.licenseNetworkError,
                                    message: "There has been a network error while obtaining the server permission!"
                                        + additionalInfo
                                }, {
                                    type: exports.LicenseErrorType.NetworkError,
                                })
                            };
                        }
                    case exports.ServerPermissionSubmitResultStatus.RemoteLock:
                        return {
                            error: new SDKError(licenseErrors.licenseRemoteLocked, {
                                type: exports.LicenseErrorType.RemoteLock,
                            }),
                            lease: serverPermission.lease
                        };
                    case exports.ServerPermissionSubmitResultStatus.PermissionExpired:
                        return {
                            error: new SDKError(licenseErrors.licensePermissionExpired, {
                                type: exports.LicenseErrorType.PermissionExpired
                            }),
                            lease: serverPermission.lease
                        };
                    case exports.ServerPermissionSubmitResultStatus.PayloadCorrupted:
                        return {
                            error: new SDKError(licenseErrors.licensePayloadCorrupted, {
                                type: exports.LicenseErrorType.PayloadCorrupted
                            }),
                            lease: serverPermission.lease
                        };
                    case exports.ServerPermissionSubmitResultStatus.PayloadSignatureVerificationFailed:
                        return {
                            error: new SDKError(licenseErrors.licensePayloadVerificationFailed, {
                                type: exports.LicenseErrorType.PayloadSignatureVerificationFailed
                            }),
                            lease: serverPermission.lease
                        };
                    case exports.ServerPermissionSubmitResultStatus.IncorrectTokenState:
                        return {
                            error: new SDKError(licenseErrors.licenseTokenStateIncorrect, {
                                type: exports.LicenseErrorType.IncorrectTokenState
                            }),
                            lease: serverPermission.lease
                        };
                }
            }
    }
}
/* eslint-enable @typescript-eslint/no-explicit-any,
                 @typescript-eslint/explicit-module-boundary-types,
                 @typescript-eslint/no-unsafe-member-access,
                 @typescript-eslint/no-unsafe-call
*/

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (DetectionStatus) {
    /** Detection has failed. */
    DetectionStatus[DetectionStatus["Failed"] = 0] = "Failed";
    /** Document has been detected. */
    DetectionStatus[DetectionStatus["Success"] = 1] = "Success";
    /** Document has been detected but the camera is too far from the document. */
    DetectionStatus[DetectionStatus["CameraTooFar"] = 2] = "CameraTooFar";
    /** Document has been detected but the camera is too close to the document. */
    DetectionStatus[DetectionStatus["CameraTooClose"] = 3] = "CameraTooClose";
    /** Document has been detected but the camera’s angle is too steep. */
    DetectionStatus[DetectionStatus["CameraAngleTooSteep"] = 4] = "CameraAngleTooSteep";
    /** Document has been detected but the document is too close to the camera edge. */
    DetectionStatus[DetectionStatus["DocumentTooCloseToCameraEdge"] = 5] = "DocumentTooCloseToCameraEdge";
    /** Only part of the document is visible. */
    DetectionStatus[DetectionStatus["DocumentPartiallyVisible"] = 6] = "DocumentPartiallyVisible";
    /** Fallback detection was successful (PhotoPay specific). */
    DetectionStatus[DetectionStatus["FallbackSuccess"] = 7] = "FallbackSuccess";
})(exports.DetectionStatus || (exports.DetectionStatus = {}));

class AbortError extends Error {
  constructor() {
    super('Throttled function aborted');
    this.name = 'AbortError';
  }
}
function pThrottle({
  limit,
  interval,
  strict
}) {
  if (!Number.isFinite(limit)) {
    throw new TypeError('Expected `limit` to be a finite number');
  }
  if (!Number.isFinite(interval)) {
    throw new TypeError('Expected `interval` to be a finite number');
  }
  const queue = new Map();
  let currentTick = 0;
  let activeCount = 0;
  function windowedDelay() {
    const now = Date.now();
    if (now - currentTick > interval) {
      activeCount = 1;
      currentTick = now;
      return 0;
    }
    if (activeCount < limit) {
      activeCount++;
    } else {
      currentTick += interval;
      activeCount = 1;
    }
    return currentTick - now;
  }
  const strictTicks = [];
  function strictDelay() {
    const now = Date.now();
    if (strictTicks.length < limit) {
      strictTicks.push(now);
      return 0;
    }
    const earliestTime = strictTicks.shift() + interval;
    if (now >= earliestTime) {
      strictTicks.push(now);
      return 0;
    }
    strictTicks.push(earliestTime);
    return earliestTime - now;
  }
  const getDelay = strict ? strictDelay : windowedDelay;
  return function_ => {
    const throttled = function (...args) {
      if (!throttled.isEnabled) {
        return (async () => function_.apply(this, args))();
      }
      let timeout;
      return new Promise((resolve, reject) => {
        const execute = () => {
          resolve(function_.apply(this, args));
          queue.delete(timeout);
        };
        timeout = setTimeout(execute, getDelay());
        queue.set(timeout, reject);
      });
    };
    throttled.abort = () => {
      for (const timeout of queue.keys()) {
        clearTimeout(timeout);
        queue.get(timeout)(new AbortError());
      }
      queue.clear();
      strictTicks.splice(0, strictTicks.length);
    };
    throttled.isEnabled = true;
    Object.defineProperty(throttled, 'queueSize', {
      get() {
        return queue.size;
      }
    });
    return throttled;
  };
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
const TARGET_FPS = 15;
const throttle = pThrottle({
    limit: 1,
    interval: Math.round((1 / TARGET_FPS) * 1000),
    strict: true,
});
(function (VideoRecognitionMode) {
    /** Normal recognition */
    VideoRecognitionMode[VideoRecognitionMode["Recognition"] = 0] = "Recognition";
    /** Indefinite scan. Useful for profiling the performance of scan (using `onDebugText` metadata callback) */
    VideoRecognitionMode[VideoRecognitionMode["RecognitionTest"] = 1] = "RecognitionTest";
    /** Only detection. Useful for profiling the performance of detection (using `onDebugText` metadata callback) */
    VideoRecognitionMode[VideoRecognitionMode["DetectionTest"] = 2] = "DetectionTest";
})(exports.VideoRecognitionMode || (exports.VideoRecognitionMode = {}));
/**
 * A wrapper around `RecognizerRunner` that can use it to perform recognition of
 * video feeds - either from live camera or from predefined video file.
 */
class VideoRecognizer {
    /**
     * **Use only if provided factory functions are not well-suited for your use
     * case.**
     *
     * Creates a new `VideoRecognizer` with provided `HTMLVideoElement`.
     *
     * Keep in mind that `HTMLVideoElement` **must have** a video feed which is
     * ready to use.
     *
     * - If you want to take advantage of provided camera management, use
     *   `createVideoRecognizerFromCameraStream`
     * - In case that static video file should be processed, use
     *   `createVideoRecognizerFromVideoPath`
     *
     * @param videoElement HTMLVideoElement with video feed which is going to be
     * processed
     * @param recognizerRunner RecognizerRunner that should be used for video
     * stream recognition
     * @param cameraFlipped Whether the camera is flipped, e.g. if front-facing
     * camera is used
     * @param deviceId
     */
    constructor(videoElement, recognizerRunner, cameraFlipped = false, deviceId = null) {
        this.deviceId = null;
        this.recognitionCancelRequested = false;
        this.recognitionPauseRequested = false;
        this.recognitionTimeoutMs = 20000;
        this.timeoutStartedAt = 0;
        this.currentTimeoutCount = 0;
        this.videoRecognitionMode = exports.VideoRecognitionMode.Recognition;
        this.onScanningDone = null;
        this.onFrameProcessed = null;
        this.cameraFlipped = false;
        this.threadBusy = false;
        this.handleFlippingVideo = async () => {
            this.videoElement.style.transform = `scaleX(${this.cameraFlipped ? -1 : 1})`;
            // needs to be set on the recognizerRunner as well to provide correct quads
            await this.recognizerRunner.setCameraPreviewMirrored(this.cameraFlipped);
        };
        /**
         * Sets the callback that will be invoked when frame is processed
         * @param onFrameProcessed Callback that will be invoked when frame is processed
         */
        this.setOnFrameProcessed = (onFrameProcessed) => {
            this.onFrameProcessed = onFrameProcessed;
        };
        this.flipCamera = async () => {
            this.cameraFlipped = !this.cameraFlipped;
            await this.handleFlippingVideo();
        };
        this.isCameraFlipped = () => {
            return this.cameraFlipped;
        };
        /**
         * Sets the video recognition mode to be used.
         */
        this.setVideoRecognitionMode = async (videoRecognitionMode) => {
            this.videoRecognitionMode = videoRecognitionMode;
            const isDetectionMode = this.videoRecognitionMode === exports.VideoRecognitionMode.DetectionTest;
            await this.recognizerRunner.setDetectionOnlyMode(isDetectionMode);
        };
        /**
         * Starts the recognition of the video stream associated with this
         * `VideoRecognizer`. The stream will be unpaused and recognition loop will
         * start. After recognition completes, an `onScanningDone` callback will be
         * invoked with state of the recognition.
         *
         * NOTE: As soon as the execution of the callback completes, the recognition
         *       loop will continue and recognition state will be retained. To clear
         *       the recognition state, use {@linkcode resetRecognizers} (within your
         *       callback). To pause the recognition loop, use
         *       {@linkcode pauseRecognition} (within your callback) - to resume it
         *       later use {@linkcode resumeRecognition}. To completely stop the
         *       recognition and video feed, while keeping the ability to use this
         *       `VideoRecognizer` later, use {@linkcode pauseVideoFeed}. To
         *       completely stop the recognition and video feed and release all the
         *       resources involved with the video stream, use
         *       {@linkcode releaseVideoFeed}.
         *
         * @param onScanningDone Callback that will be invoked when recognition
         * completes.
         * @param recognitionTimeoutMs Amount of time in ms that the recognizer will
         * stay in the `Uncertain` state before resolving.
         */
        this.startRecognition = async (onScanningDone, recognitionTimeoutMs = 20000) => {
            try {
                await this.videoElement.play();
            }
            catch (error) {
                throw new Error(exports.ErrorMessages.VIDEO_RECOGNIZER_PLAY_REQUEST_INTERRUPTED);
            }
            // Following 2 lines might not be needed. Just in case left here.
            this.recognitionPauseRequested = false;
            this.recognitionCancelRequested = false;
            this.clearTimeout();
            this.recognitionTimeoutMs = recognitionTimeoutMs;
            this.onScanningDone = onScanningDone;
            await this.throttledQueueFrame();
        };
        /**
         * Resumes the recognition and video playback
         * @param resetRecognizers Indicates whether resetRecognizers should be
         * invoked while resuming the recognition
         */
        this.resumeRecognition = async (resetRecognizers) => {
            if (resetRecognizers) {
                try {
                    await this.resetRecognizers(true);
                }
                catch (error) {
                    throw new SDKError(videoRecognizerErrors.recognizersResetFailure);
                }
            }
            try {
                await this.videoElement.play();
            }
            catch (error) {
                throw new Error(exports.ErrorMessages.VIDEO_RECOGNIZER_PLAY_REQUEST_INTERRUPTED);
            }
            try {
                this.recognitionPauseRequested = false;
                await this.throttledQueueFrame();
            }
            catch (error) {
                this.recognitionPauseRequested = true;
                console.error(error);
            }
        };
        /**
         * Performs the recognition of the video stream associated with this
         * `VideoRecognizer`. The stream will be unpaused, recognition will be
         * performed and promise will be resolved with recognition status. After the
         * resolution of returned promise, the video stream will be paused, but not
         * released. To release the stream, use function `releaseVideoFeed`.
         *
         * This is a simple version of {@linkcode startRecognition} that should be
         * used for most cases, like when you only need to perform one scan per video
         * session.
         *
         * @param recognitionTimeoutMs Amount of time in ms that the recognizer will
         * stay in the `Uncertain` state before resolving.
         */
        this.recognize = (recognitionTimeoutMs = 20000) => {
            return new Promise((resolve) => {
                const onScanningDone = (recognitionState) => {
                    this.pauseVideoFeed();
                    resolve(recognitionState);
                };
                void this.startRecognition(onScanningDone, recognitionTimeoutMs);
            });
        };
        /**
         * Pauses the video feed. You can resume the feed by calling recognize or
         * `startRecognition`. Note that this pauses both the camera feed and
         * recognition. If you just want to pause recognition, while keeping the
         * camera feed active, call method `pauseRecognition`.
         */
        this.pauseVideoFeed = () => {
            // fix for https://developer.chrome.com/blog/play-request-was-interrupted/
            if (this.videoElement.readyState >
                this.videoElement.HAVE_CURRENT_DATA &&
                !this.videoElement.paused) {
                this.videoElement.pause();
                this.pauseRecognition();
            }
        };
        /**
         * Pauses the recognition. This means that video frames that arrive from given
         * video source will not be recognized. To resume recognition, call
         * {@linkcode resumeRecognition}.
         */
        this.pauseRecognition = () => {
            this.recognitionPauseRequested = true;
        };
        /**
         * Cancels current ongoing recognition. Unlike {@linkcode pauseRecognition} this will reset everything
         */
        this.cancelRecognition = () => {
            this.recognitionCancelRequested = true;
        };
        /**
         * Convenience method for invoking
         * {@linkcode RecognizerRunner.resetRecognizers} on associated
         * `RecognizerRunner`.
         */
        this.resetRecognizers = async (hardReset) => {
            await this.recognizerRunner.resetRecognizers(hardReset);
        };
        /**
         * Convenience method for accessing `RecognizerRunner` associated with this
         * `VideoRecognizer`. Sometimes it's useful to reconfigure `RecognizerRunner`
         * while handling `onScanningDone` callback and this method makes that much
         * more convenient.
         */
        this.getRecognizerRunner = () => {
            return this.recognizerRunner;
        };
        /**
         * Getter for {@linkcode videoElement}
         */
        this.getVideoElement = () => {
            return this.videoElement;
        };
        /**
         * Change currently used camera device for recognition. To get list of
         * available camera devices use `getCameraDevices` method.
         *
         * Keep in mind that this method will reset recognizers.
         *
         * @param camera Desired camera device which should be used for recognition.
         */
        this.changeCameraDevice = async (camera) => {
            this.pauseRecognition();
            this.releaseVideoFeed();
            await bindCameraToVideoFeed(camera, this.videoElement, exports.PreferredCameraType.BackFacingCamera);
            await this.resumeRecognition(true);
        };
        /**
         * Shorthand for queuing the next frame for processing. Wrapper around
         * {@linkcode recognitionLoop}. Resolves when the frame is done processing.
         */
        this.queueFrame = () => {
            // promisify `requestVideoFrameCallback` so that we know when it triggers
            return new Promise((resolve) => {
                this.frameCallback(() => {
                    void this.recognitionLoop().then(() => resolve());
                });
            });
        };
        this.throttledQueueFrame = throttle(this.queueFrame);
        /**
         * The main loop. Takes camera frames from {@linkcode videoElement} and
         * processes them on the `recognizerRunner`.
         */
        this.recognitionLoop = async () => {
            if (this.threadBusy) {
                return;
            }
            // exit without side-effects when paused
            if (this.recognitionPauseRequested) {
                return;
            }
            // if cancelled exit and reset `VideoRecognizer` state
            if (this.recognitionCancelRequested) {
                this.clearTimeout();
                await this.resetRecognizers(true);
                this.onScanningDone = null;
                this.recognitionCancelRequested = false;
                return;
            }
            /*
            Start processing.
    
            At this point we draw the canvas frames, extract the ImageData` and send it
            to the `RecognizerRunner`. The main thread and the worker thread should be
            treated as blocked.
            */
            const cameraFrame = captureFrame(this.videoElement);
            // queue everything below in a macrotask
            await new Promise((f) => setTimeout(f, 0));
            this.threadBusy = true;
            const processResult = await this.recognizerRunner.processImage(cameraFrame);
            // assumption: only one recognizer is used
            const currentFrameResult = await this.recognizerRunner.recognizers[0].getResult();
            this.threadBusy = false;
            // Trigger onFrameProcessed callback
            if (typeof this.onFrameProcessed === "function") {
                this.onFrameProcessed(currentFrameResult, cameraFrame.imageData);
            }
            // End processing
            // Test mode resets recognizers on every tick and never times out
            if (this.videoRecognitionMode === exports.VideoRecognitionMode.DetectionTest ||
                this.videoRecognitionMode === exports.VideoRecognitionMode.RecognitionTest) {
                await this.recognizerRunner.resetRecognizers(true);
                this.clearTimeout();
                void this.throttledQueueFrame();
                return;
            }
            // regular flow
            switch (processResult) {
                // `Valid` stops loop and calls `onScanningDone`
                case exports.RecognizerResultState.Valid: {
                    this.clearTimeout();
                    if (typeof this.onScanningDone === "function") {
                        this.onScanningDone(processResult);
                    }
                    return;
                }
                // `Uncertain` resolves after a timeout, loops otherwise
                case exports.RecognizerResultState.Uncertain: {
                    // increment timeout
                    const now = performance.now();
                    if (this.timeoutStartedAt === 0) {
                        this.timeoutStartedAt = now;
                    }
                    this.currentTimeoutCount = now - this.timeoutStartedAt;
                    // if under timeout continue looping
                    if (this.currentTimeoutCount < this.recognitionTimeoutMs) {
                        void this.throttledQueueFrame();
                        return;
                    }
                    // otherwise stop as `Uncertain`
                    this.clearTimeout();
                    if (typeof this.onScanningDone === "function") {
                        this.onScanningDone(processResult);
                    }
                    return;
                }
                // `StageValid` and `Empty` loop forever
                case exports.RecognizerResultState.StageValid:
                case exports.RecognizerResultState.Empty: {
                    this.clearTimeout();
                    void this.throttledQueueFrame();
                    return;
                }
            }
        };
        /**
         * Clear timeout on every loop which didn't result in `RecognizerResultState.Uncertain`
         */
        this.clearTimeout = () => {
            this.currentTimeoutCount = 0;
            this.timeoutStartedAt = 0;
        };
        /**
         * Stops all media stream tracks associated with {@linkcode videoElement}.
         */
        this.releaseVideoFeed = () => {
            if (this.videoElement &&
                this.videoElement.srcObject !== null &&
                this.videoElement.srcObject instanceof MediaStream) {
                this.videoElement.srcObject
                    .getTracks()
                    .forEach((track) => track.stop());
                this.videoElement.srcObject = null;
            }
        };
        this.videoElement = videoElement;
        this.recognizerRunner = recognizerRunner;
        this.cameraFlipped = cameraFlipped;
        this.deviceId = deviceId;
        if ("requestVideoFrameCallback" in HTMLVideoElement.prototype) {
            this.frameCallback =
                this.videoElement.requestVideoFrameCallback.bind(this.videoElement);
        }
        else {
            this.frameCallback = window.requestAnimationFrame.bind(window);
        }
        void this.handleFlippingVideo();
        // Prepare the `video` element so that it can autoplay on iOS
        // https://webkit.org/blog/7734/auto-play-policy-changes-for-macos/
        // https://developer.mozilla.org/en-US/docs/Web/Media/Autoplay_guide#autoplay_availability
        this.videoElement.setAttribute("playsinline", "");
        this.videoElement.setAttribute("mute", "");
    }
    /**
     * Creates a new VideoRecognizer by opening a camera stream and attaching it
     * to given `HTMLVideoElement`. If camera cannot be accessed, the returned
     * promise will be rejected.
     *
     * @param cameraFeed `HTMLVideoElement` to which camera stream should be
     * attached
     * @param recognizerRunner `RecognizerRunner` that should be used for video
     * stream recognition
     * @param cameraId User can provide specific camera ID to be selected and used
     * @param preferredCameraType Whether back facing or front facing camera is
     *        preferred. Obeyed only if there is a choice (i.e. if device has only
     *        front-facing camera, the opened camera will be a front-facing
     *        camera, regardless of preference)
     */
    static async createVideoRecognizerFromCameraStream(cameraFeed, recognizerRunner, cameraId = null, preferredCameraType = exports.PreferredCameraType.BackFacingCamera) {
        if (!cameraFeed || !(cameraFeed instanceof HTMLVideoElement)) {
            throw new SDKError(videoRecognizerErrors.elementMissing);
        }
        if (!navigator.mediaDevices.getUserMedia) {
            throw new SDKError(videoRecognizerErrors.mediaDevicesUnsupported);
        }
        const selectedCamera = await selectCamera(cameraId, preferredCameraType);
        if (!selectedCamera) {
            throw new SDKError(videoRecognizerErrors.cameraMissing);
        }
        const cameraFlipped = await bindCameraToVideoFeed(selectedCamera, cameraFeed, preferredCameraType);
        return new VideoRecognizer(cameraFeed, recognizerRunner, cameraFlipped, selectedCamera.deviceId);
    }
    /**
     * Creates a new `VideoRecognizer` by attaching the given URL to video to
     * given `HTMLVideoElement` and using it to display video frames while
     * processing them.
     *
     * @param videoPath URL of the video file that should be recognized.
     * @param videoElement `HTMLVideoElement` to which video file will be attached
     * @param recognizerRunner `RecognizerRunner` that should be used for video
     * stream recognition.
     */
    static createVideoRecognizerFromVideoPath(videoPath, videoElement, recognizerRunner) {
        const videoRecognizer = new VideoRecognizer(videoElement, recognizerRunner);
        videoElement.src = videoPath;
        videoElement.currentTime = 0;
        videoElement.onended = () => {
            videoRecognizer.cancelRecognition();
        };
        return videoRecognizer;
    }
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
/**
 * Settings object for function loadWasmModule.
 */
class WasmSDKLoadSettings {
    /**
     * @param licenseKey License key for unlocking the WebAssembly module.
     */
    constructor(licenseKey) {
        /**
         * Write a hello message to the browser console when license check is successfully performed.
         *
         * Hello message will contain the name and version of the SDK, which are required information for all support
         * tickets.
         *
         * The default value is true.
         */
        this.allowHelloMessage = true;
        /**
         * Absolute location of WASM and related JS/data files. Useful when resource files should be loaded over CDN, or
         * when web frameworks/libraries are used which store resources in specific locations, e.g. inside "assets" folder.
         *
         * Important: if the engine is hosted on another origin, CORS must be enabled between two hosts. That is, server
         * where engine is hosted must have 'Access-Control-Allow-Origin' header for the location of the web app.
         *
         * Important: SDK and WASM resources must be from the same version of a package.
         *
         * Default value is empty string, i.e. "". In case of empty string, value of "window.location.origin" property is
         * going to be used.
         */
        this.engineLocation = "";
        /**
         * The absolute location of the Web Worker script file that loads the WebAssembly module.
         *
         * Important: the worker script must be served via HTTPS and must be of the same origin as the initiator.
         * See https://github.com/w3c/ServiceWorker/issues/940 (same applies for Web Workers).
         *
         * Important: SDK, worker script and WebAssembly resources must be from the same version of the package.
         *
         * The default value is an empty string, i.e. "", and in that case, the worker script is loaded from the default
         * location in resources folder.
         */
        this.workerLocation = "";
        /**
         * Type of the WASM that will be loaded. By default, if not set, the SDK will automatically determine the best WASM
         * to load.
         */
        this.wasmType = null;
        /**
         * Flavor of the WASM that will be loaded. By default, the SDK will automatically determine the best
         * WASM flavor to load.
         * If there are no flavors, use "", i.e. empty string.
         */
        this.wasmFlavor = defaultWasmFlavor();
        /**
         * Defines the number of workers that will be used for multi-threaded processing of the images. If not set, the
         * number of worker used will match the number of detected CPU cores on a device.
         *
         * If the browser does not support multi-threaded processing or it was deliberately disabled using the `wasmType`
         * property, then this property will be ignored.
         */
        this.numberOfWorkers = null;
        /**
         * Optional callback function that will report the SDK loading progress.
         *
         * This can be useful for displaying progress bar to users with slow connections.
         *
         * The default value is null.
         */
        this.loadProgressCallback = null;
        /**
         * Name of the file containing the WebAssembly module.
         *
         * Change this only if you have renamed the original WASM and its support JS file for your purposes.
         */
        this.wasmModuleName = defaultWasmModuleName;
        if (!licenseKey) {
            throw new SDKError(sdkErrors.licenseKeyMissing);
        }
        this.licenseKey = licenseKey;
    }
}

const bulkMemory = async () => WebAssembly.validate(new Uint8Array([0, 97, 115, 109, 1, 0, 0, 0, 1, 4, 1, 96, 0, 0, 3, 2, 1, 0, 5, 3, 1, 0, 1, 10, 14, 1, 12, 0, 65, 0, 65, 0, 65, 0, 252, 10, 0, 0, 11])),
  mutableGlobals = async () => WebAssembly.validate(new Uint8Array([0, 97, 115, 109, 1, 0, 0, 0, 2, 8, 1, 1, 97, 1, 98, 3, 127, 1, 6, 6, 1, 127, 1, 65, 0, 11, 7, 5, 1, 1, 97, 3, 1])),
  referenceTypes = async () => WebAssembly.validate(new Uint8Array([0, 97, 115, 109, 1, 0, 0, 0, 1, 4, 1, 96, 0, 0, 3, 2, 1, 0, 10, 7, 1, 5, 0, 208, 112, 26, 11])),
  saturatedFloatToInt = async () => WebAssembly.validate(new Uint8Array([0, 97, 115, 109, 1, 0, 0, 0, 1, 4, 1, 96, 0, 0, 3, 2, 1, 0, 10, 12, 1, 10, 0, 67, 0, 0, 0, 0, 252, 0, 26, 11])),
  signExtensions = async () => WebAssembly.validate(new Uint8Array([0, 97, 115, 109, 1, 0, 0, 0, 1, 4, 1, 96, 0, 0, 3, 2, 1, 0, 10, 8, 1, 6, 0, 65, 0, 192, 26, 11])),
  simd = async () => WebAssembly.validate(new Uint8Array([0, 97, 115, 109, 1, 0, 0, 0, 1, 5, 1, 96, 0, 1, 123, 3, 2, 1, 0, 10, 10, 1, 8, 0, 65, 0, 253, 15, 253, 98, 11])),
  threads = () => (async e => {
    try {
      return "undefined" != typeof MessageChannel && new MessageChannel().port1.postMessage(new SharedArrayBuffer(1)), WebAssembly.validate(e);
    } catch (e) {
      return !1;
    }
  })(new Uint8Array([0, 97, 115, 109, 1, 0, 0, 0, 1, 4, 1, 96, 0, 0, 3, 2, 1, 0, 5, 4, 1, 3, 1, 1, 10, 11, 1, 9, 0, 65, 0, 254, 16, 2, 0, 26, 11]));

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
function isSafari() {
    const userAgent = navigator.userAgent.toLowerCase();
    return userAgent.includes("safari") && !userAgent.includes("chrome");
}
function isIOSUserAgent() {
    const pattern = /iOS|iPhone|iPad|iPod/i; // 'i' flag for case-insensitive matching
    return pattern.test(navigator.userAgent);
}
/* eslint-disable max-len */
/**
 * Safari 16 shipped with WASM threads support, but it didn't ship with nested
 * workers support, so an extra check is needed
 * https://github.com/GoogleChromeLabs/squoosh/pull/1325/files#diff-904900db64cd3f48b0e765dbbdc6a218a7ea74a199671bde82a8944a904db86f
 */
/* eslint-enable max-len */
async function checkThreadsSupport() {
    const supportsWasmThreads = await threads();
    if (!supportsWasmThreads)
        return false;
    if (!("importScripts" in self)) {
        throw Error("Not implemented");
    }
    // Safari has issues with shared memory
    // https://github.com/emscripten-core/emscripten/issues/19374
    if (isSafari()) {
        return false;
    }
    return "Worker" in self;
}
async function detectWasmFeatures() {
    const basicSet = [
        mutableGlobals(),
        referenceTypes(),
        bulkMemory(),
        saturatedFloatToInt(),
        signExtensions(),
    ];
    const supportsBasic = (await Promise.all(basicSet)).every(Boolean);
    if (!supportsBasic) {
        throw new Error("Browser doesn't meet minimum requirements!");
    }
    const supportsAdvanced = await simd();
    if (!supportsAdvanced) {
        return exports.WasmType.Basic;
    }
    const supportsAdvancedThreads = await checkThreadsSupport();
    if (!supportsAdvancedThreads) {
        return exports.WasmType.Advanced;
    }
    return exports.WasmType.AdvancedWithThreads;
}
async function detectWasmType() {
    // determine if all features required for advanced WASM are available
    // currently, advanced wasm requires SIMD
    const haveSIMD = await simd();
    const threadsSupported = await checkThreadsSupport();
    if (haveSIMD) {
        if (threadsSupported) {
            return exports.WasmType.AdvancedWithThreads;
        }
        else {
            return exports.WasmType.Advanced;
        }
    }
    else {
        return exports.WasmType.Basic;
    }
}
function wasmFolder(wasmType) {
    switch (wasmType) {
        case exports.WasmType.AdvancedWithThreads:
            return "advanced-threads";
        case exports.WasmType.Advanced:
            return "advanced";
        case exports.WasmType.Basic:
            return "basic";
    }
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
// taken from https://stackoverflow.com/a/2117523/213057
/* eslint-disable */
function uuidv4() {
    return ([1e7] + -1e3 + -4e3 + -8e3 + -1e11).replace(/[018]/g, (c) => (c ^
        (crypto.getRandomValues(new Uint8Array(1))[0] & (15 >> (c / 4)))).toString(16));
}
/* eslint-enable */
function getUserID() {
    try {
        let userId = localStorage.getItem("mb-user-id");
        if (userId === null) {
            userId = uuidv4();
            localStorage.setItem("mb-user-id", userId);
        }
        return userId;
    }
    catch (error) {
        // local storage is disabled, generate new user ID every time
        return uuidv4();
    }
}
/**
 * Asynchronously loads and compiles the WebAssembly module.
 * @param loadSettings Object defining the settings for loading the WebAssembly module.
 * @returns Promise that resolves if WebAssembly module was successfully loaded and rejects if not.
 */
/* eslint-disable @typescript-eslint/no-explicit-any,
                  @typescript-eslint/no-unsafe-assignment,
                  @typescript-eslint/no-unsafe-member-access,
                  @typescript-eslint/no-unsafe-call */
async function loadWasmModule(loadSettings) {
    return new Promise((resolve, reject) => {
        if (!loadSettings || typeof loadSettings !== "object") {
            reject(new SDKError(sdkErrors.wasmSettingsMissing));
            return;
        }
        if (typeof loadSettings.licenseKey !== "string") {
            reject(new SDKError(sdkErrors.licenseKeyMissing));
            return;
        }
        if (!loadSettings.wasmModuleName) {
            reject(new SDKError(sdkErrors.wasmModuleNameMissing));
            return;
        }
        if (typeof loadSettings.engineLocation !== "string") {
            reject(new SDKError(sdkErrors.engineLocationInvalid));
            return;
        }
        if (typeof loadSettings.workerLocation !== "string") {
            reject(new SDKError(sdkErrors.workerLocationInvalid));
            return;
        }
        // obtain user ID from local storage
        const userId = getUserID();
        try {
            const workerPath = `/resources/${loadSettings.wasmModuleName}.worker.min.js`;
            const defaultWorkerLocation = window.location.origin + workerPath;
            const workerLocation = loadSettings.workerLocation || defaultWorkerLocation;
            if (loadSettings.allowHelloMessage) {
                console.log("Worker location is:", workerLocation);
            }
            const worker = new Worker(workerLocation);
            WasmSDKWorker.createWasmWorker(worker, loadSettings, userId).then((wasmSDK) => {
                resolve(wasmSDK);
            }, reject);
        }
        catch (initError) {
            reject(initError);
        }
    });
}
/* eslint-enable @typescript-eslint/no-explicit-any,
                 @typescript-eslint/no-unsafe-assignment,
                 @typescript-eslint/no-unsafe-member-access,
                 @typescript-eslint/no-unsafe-call */
/**
 * Function for creating a new RecognizerRunner.
 * Note that it is currently not possible to have multiple instances of RecognizerRunner per instance of WasmSDK.
 * Attempt to create new instance of RecognizerRunner prior deleting the previous one will fail.
 * @param wasmSDK Instance of WasmSDK which will be used to communicate with the WebAssembly module.
 * @param recognizers Array of recognizers that will be used by RecognizerRunner.
 * @param allowMultipleResults Whether or not it is allowed to return multiple results from single recognition session.
 *        See README.md for more information.
 * @param metadataCallbacks
 */
async function createRecognizerRunner(wasmSDK, recognizers, allowMultipleResults = false, metadataCallbacks = {}) {
    if (typeof wasmSDK !== "object") {
        throw new SDKError(sdkErrors.missing);
    }
    if (typeof recognizers !== "object" || recognizers.length < 1) {
        throw new SDKError(sdkErrors.recognizersMissing);
    }
    return wasmSDK.mbWasmModule.createRecognizerRunner(recognizers, allowMultipleResults, metadataCallbacks);
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
/**
 * Creates a new instance of `SuccessFrameGrabberRecognizer`.
 * @param wasmSDK Instance of WasmSDK which will be used to communicate with the
 *        WebAssembly module.
 * @param slaveRecognizer Instance of Recognizer that will be wrapped.
 */
async function createSuccessFrameGrabberRecognizer(wasmSDK, slaveRecognizer) {
    // taken from https://stackoverflow.com/a/53615996
    const sfgr = await wasmSDK.mbWasmModule.newRecognizer("SuccessFrameGrabberRecognizer", slaveRecognizer);
    const mutableSFGR = sfgr;
    mutableSFGR.wrappedRecognizer = slaveRecognizer;
    return sfgr;
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
/**
 * Extension factors relative to corresponding dimension of the full image. For example,
 * {@code upFactor} and {@code downFactor} define extensions relative to image height, e.g.
 * when {@code upFactor} is 0.5, upper image boundary will be extended for half of image's full
 * height.
 *
 *                      ._______________________________________.
 *                      |                   ↑                   |
 *                      |                upFactor               |
 *   .________.         |              .________.               |
 *   |        |   -->   |  ⃖ leftFactor |        | rightFactor  ⃗ |
 *   |________|         |              |________|               |
 *                      |                   ↓                   |
 *                      |               downFactor              |
 *                      |_______________________________________|
 *
 */
class ExtensionFactors {
    /**
     * Constructor which accepts image extension factors which must be in range [-1.0f, 1.0f].
     * @param upFactor image extension factor relative to full image height in UP direction
     * @param downFactor image extension factor relative to full image height in DOWN direction
     * @param leftFactor image extension factor relative to full image width in LEFT direction
     * @param rightFactor image extension factor relative to full image width in RIGHT direction
     */
    constructor(upFactor = 0.0, downFactor = 0.0, leftFactor = 0.0, rightFactor = 0.0) {
        /**
         * Currently used image extension factor relative to full image height in UP direction.
         */
        this.upFactor = 0.0;
        /**
         * Currently used image extension factor relative to full image height in DOWN direction.
         */
        this.downFactor = 0.0;
        /**
         * Currently used image extension factor relative to full image height in LEFT direction.
         */
        this.leftFactor = 0.0;
        /**
         * Currently used image extension factor relative to full image height in RIGHT direction.
         */
        this.rightFactor = 0.0;
        this.checkExtensionFactor(upFactor);
        this.checkExtensionFactor(downFactor);
        this.checkExtensionFactor(leftFactor);
        this.checkExtensionFactor(rightFactor);
        this.upFactor = upFactor;
        this.downFactor = downFactor;
        this.leftFactor = leftFactor;
        this.rightFactor = rightFactor;
    }
    checkExtensionFactor(factor) {
        if (factor > 1.0 || factor < -1.0) {
            throw new Error("Extension factor must be in range [-1.0, 1.0]");
        }
    }
}
function validateDpi(dpi) {
    if (dpi < 100 || dpi > 400) {
        throw new Error("DPI must be from interval [100, 400]");
    }
}

/**
 * Copyright (c) Microblink Ltd. All rights reserved.
 */
(function (LivenessStatus) {
    /**
     *  Verification process has not been started yet
     */
    LivenessStatus[LivenessStatus["NotAvailable"] = 0] = "NotAvailable";
    /**
     * Hand not present
     */
    LivenessStatus[LivenessStatus["HandNotPresent"] = 1] = "HandNotPresent";
    /**
     * Not enough hand present
     */
    LivenessStatus[LivenessStatus["NotEnoughHandPresent"] = 2] = "NotEnoughHandPresent";
    /**
     * Detected document not in hand
     */
    LivenessStatus[LivenessStatus["DocumentNotInHand"] = 3] = "DocumentNotInHand";
    /**
     * Detected document in hand
     */
    LivenessStatus[LivenessStatus["DocumentInHand"] = 4] = "DocumentInHand";
})(exports.LivenessStatus || (exports.LivenessStatus = {}));
(function (CheckResult) {
    CheckResult[CheckResult["NotPerformed"] = 0] = "NotPerformed";
    CheckResult[CheckResult["Pass"] = 1] = "Pass";
    CheckResult[CheckResult["Fail"] = 2] = "Fail";
})(exports.CheckResult || (exports.CheckResult = {}));
(function (MatchLevel) {
    MatchLevel[MatchLevel["Disabled"] = 0] = "Disabled";
    MatchLevel[MatchLevel["Level1"] = 1] = "Level1";
    MatchLevel[MatchLevel["Level2"] = 2] = "Level2";
    MatchLevel[MatchLevel["Level3"] = 3] = "Level3";
    MatchLevel[MatchLevel["Level4"] = 4] = "Level4";
    MatchLevel[MatchLevel["Level5"] = 5] = "Level5";
    MatchLevel[MatchLevel["Level6"] = 6] = "Level6";
    MatchLevel[MatchLevel["Level7"] = 7] = "Level7";
    MatchLevel[MatchLevel["Level8"] = 8] = "Level8";
    MatchLevel[MatchLevel["Level9"] = 9] = "Level9";
    MatchLevel[MatchLevel["Level10"] = 10] = "Level10";
})(exports.MatchLevel || (exports.MatchLevel = {}));
(function (AnonymizationMode) {
    /** No anonymization is performed in this mode. */
    AnonymizationMode[AnonymizationMode["None"] = 0] = "None";
    /**
     * Sensitive data in the document image is anonymized with black boxes covering selected sensitive data. Data
     * returned in result fields is not changed.
     */
    AnonymizationMode[AnonymizationMode["ImageOnly"] = 1] = "ImageOnly";
    /** Document image is not changed. Data returned in result fields is redacted. */
    AnonymizationMode[AnonymizationMode["ResultFieldsOnly"] = 2] = "ResultFieldsOnly";
    /**
     * Sensitive data in the image is anonymized with black boxes covering selected sensitive data. Data returned in
     * result fields is redacted.
     */
    AnonymizationMode[AnonymizationMode["FullResult"] = 3] = "FullResult";
    /** Number of possible anonymization modes. */
    AnonymizationMode[AnonymizationMode["Count"] = 4] = "Count";
})(exports.AnonymizationMode || (exports.AnonymizationMode = {}));
/**
 * Holds the settings which control card number anonymization.
 */
class CardNumberAnonymizationSettings {
    constructor() {
        /** Defines the mode of card number anonymization. */
        this.mode = exports.AnonymizationMode.ImageOnly;
        /** Defines how many digits at the beginning of the card number remain visible after anonymization. */
        this.prefixDigitsVisible = 0;
        /** Defines how many digits at the end of the card number remain visible after anonymization. */
        this.suffixDigitsVisible = 4;
    }
}
class AnonymizationSettings {
    constructor() {
        /** Defines the parameters of card number anonymization. */
        this.cardNumberAnonymizationSettings = new CardNumberAnonymizationSettings();
        /** Defines the mode of card number prefix anonymization. */
        this.cardNumberPrefixAnonymizationMode = exports.AnonymizationMode.None;
        /** Defines the mode of CVV anonymization. */
        this.cvvAnonymizationMode = exports.AnonymizationMode.None;
        /** Defines the mode of IBAN anonymization. */
        this.ibanAnonymizationMode = exports.AnonymizationMode.ImageOnly;
        /** Defines the mode of owner anonymization. */
        this.ownerAnonymizationMode = exports.AnonymizationMode.None;
        /** If true, anonymization is applied on all fields of the image if extraction is uncertain. */
        this.fallbackAnonymization = false;
    }
}
/**
 * A settings object that is used for configuring the BlinkCardRecognizer.
 */
class BlinkCardRecognizerSettings {
    constructor() {
        /**
         * Whether blured frames filtering is allowed
         */
        this.allowBlurFilter = true;
        /** Whether sensitive data should be redacted from the result */
        this.anonymizationSettings = new AnonymizationSettings();
        /**
         * Padding is a minimum distance from the edge of the frame and is defined
         * as a percentage of the frame width. Default value is 0.0f and in that case
         * padding edge and image edge are the same.
         * If padding edge is needed, recommended value is 0.02f.
         */
        this.paddingEdge = 0.0;
        /**
         * Whether invalid card number is accepted
         */
        this.allowInvalidCardNumber = false;
        /**
         * Whether to return CVV in recognizer results
         */
        this.extractCvv = true;
        /**
         * Whether to return expiry date in recognizer results
         */
        this.extractExpiryDate = true;
        /**
         * Whether to return IBAN in recognizer results
         */
        this.extractIban = true;
        /**
         * Whether to return owner in recognizer results
         */
        this.extractOwner = true;
        // implementation of the FullDocumentImageOptions interface
        this.returnFullDocumentImage = true;
        this.returnEncodedFullDocumentImage = false;
        /**
         * Hand scale is calculated as a ratio between area of hand mask and document mask. `handScaleThreshold` is
         * the minimal value for hand to be accepted as significant.
         */
        this.handScaleThreshold = 0.15;
        /**
         *  This parameter is used to adjust heuristics that eliminate cases when the hand is present in
            the input but it is not holding the document. `handDocumentOverlapThreshold` is the minimal ratio of hand
            pixels inside the frame surrounding the document and area of that frame. Only pixels inside that frame are
            used to ignore false-positive hand segmentations inside the document (e.g. profile photo on documents)
         */
        this.handDocumentOverlapThreshold = 0.05;
        /**
         * Screen analysis match level - higher is stricter.
         */
        this.screenAnalysisMatchLevel = exports.MatchLevel.Level5;
        /**
         * Photocopy analysis match level - higher is stricter.
         */
        this.photocopyAnalysisMatchLevel = exports.MatchLevel.Level5;
        /**
         * Called when document verification status is available.
         */
        this.documentLivenessCallback = null;
        this._fullDocumentImageDpi = 250;
        this.fullDocumentImageExtensionFactors = new ExtensionFactors();
    }
    get fullDocumentImageDpi() { return this._fullDocumentImageDpi; }
    set fullDocumentImageDpi(value) {
        validateDpi(value);
        this._fullDocumentImageDpi = value;
    }
}
/**
 * This function is used to create a new instance of `BlinkCardRecognizer`.
 * @param wasmSDK Instance of WasmSDK which will be used to communicate with the WebAssembly module.
 */
async function createBlinkCardRecognizer(wasmSDK) {
    return wasmSDK.mbWasmModule.newRecognizer("BlinkCardRecognizer");
}
(function (CardIssuer) {
    /* Unidentified Card */
    CardIssuer[CardIssuer["Other"] = 0] = "Other";
    /* The American Express Company Card */
    CardIssuer[CardIssuer["AmericanExpress"] = 1] = "AmericanExpress";
    /* China UnionPay Card */
    CardIssuer[CardIssuer["ChinaUnionPay"] = 2] = "ChinaUnionPay";
    /* Diners Club International Card */
    CardIssuer[CardIssuer["Diners"] = 3] = "Diners";
    /* Discover Card */
    CardIssuer[CardIssuer["DiscoverCard"] = 4] = "DiscoverCard";
    /*  Elo card association */
    CardIssuer[CardIssuer["Elo"] = 5] = "Elo";
    /* The JCB Company Card */
    CardIssuer[CardIssuer["Jcb"] = 6] = "Jcb";
    /* Maestro Debit Card */
    CardIssuer[CardIssuer["Maestro"] = 7] = "Maestro";
    /* Mastercard Inc. Card */
    CardIssuer[CardIssuer["Mastercard"] = 8] = "Mastercard";
    /* RuPay */
    CardIssuer[CardIssuer["RuPay"] = 9] = "RuPay";
    /* Interswitch Verve Card */
    CardIssuer[CardIssuer["Verve"] = 10] = "Verve";
    /* Visa Inc. Card */
    CardIssuer[CardIssuer["Visa"] = 11] = "Visa";
    /* V Pay */
    CardIssuer[CardIssuer["VPay"] = 12] = "VPay";
    /* Enum count */
    CardIssuer[CardIssuer["Count"] = 13] = "Count";
})(exports.CardIssuer || (exports.CardIssuer = {}));
(function (ProcessingStatus) {
    /** Recognition was successful. */
    ProcessingStatus[ProcessingStatus["Success"] = 0] = "Success";
    /** Detection of the document failed. */
    ProcessingStatus[ProcessingStatus["DetectionFailed"] = 1] = "DetectionFailed";
    /** Preprocessing of the input image has failed. */
    ProcessingStatus[ProcessingStatus["ImagePreprocessingFailed"] = 2] = "ImagePreprocessingFailed";
    /** Recognizer has inconsistent results. */
    ProcessingStatus[ProcessingStatus["StabilityTestFailed"] = 3] = "StabilityTestFailed";
    /** Wrong side of the document has been scanned. */
    ProcessingStatus[ProcessingStatus["ScanningWrongSide"] = 4] = "ScanningWrongSide";
    /** Identification of the fields present on the card has failed. */
    ProcessingStatus[ProcessingStatus["FieldIdentificationFailed"] = 5] = "FieldIdentificationFailed";
    /** Failed to return a requested image. */
    ProcessingStatus[ProcessingStatus["ImageReturnFailed"] = 6] = "ImageReturnFailed";
    /** Payment card currently not supported by the recognizer. */
    ProcessingStatus[ProcessingStatus["UnsupportedCard"] = 7] = "UnsupportedCard";
    /** Number of possible processing statuses. */
    ProcessingStatus[ProcessingStatus["Count"] = 8] = "Count";
})(exports.ProcessingStatus || (exports.ProcessingStatus = {}));

exports.AnonymizationSettings = AnonymizationSettings;
exports.BlinkCardRecognizerSettings = BlinkCardRecognizerSettings;
exports.CapturedFrame = CapturedFrame;
exports.CardNumberAnonymizationSettings = CardNumberAnonymizationSettings;
exports.SDKError = SDKError;
exports.SelectedCamera = SelectedCamera;
exports.SerializableSDKError = SerializableSDKError;
exports.VideoRecognizer = VideoRecognizer;
exports.WasmSDKLoadSettings = WasmSDKLoadSettings;
exports.bindCameraToVideoFeed = bindCameraToVideoFeed;
exports.captureFrame = captureFrame;
exports.createBlinkCardRecognizer = createBlinkCardRecognizer;
exports.createRecognizerRunner = createRecognizerRunner;
exports.createSuccessFrameGrabberRecognizer = createSuccessFrameGrabberRecognizer;
exports.detectWasmFeatures = detectWasmFeatures;
exports.detectWasmType = detectWasmType;
exports.frameCaptureErrors = frameCaptureErrors;
exports.getCameraDevices = getCameraDevices;
exports.isAndroidDevice = isAndroidDevice;
exports.isBrowserSupported = isBrowserSupported;
exports.isIOSDevice = isIOSDevice;
exports.isIOSUserAgent = isIOSUserAgent;
exports.isInAppBrowser = isInAppBrowser;
exports.isMobileDevice = isMobileDevice;
exports.isSafari = isSafari;
exports.licenseErrors = licenseErrors;
exports.loadWasmModule = loadWasmModule;
exports.localSdkErrors = localSdkErrors;
exports.obtainNewServerPermission = obtainNewServerPermission;
exports.sdkErrors = sdkErrors;
exports.selectCamera = selectCamera;
exports.unlockWasmSDK = unlockWasmSDK;
exports.videoRecognizerErrors = videoRecognizerErrors;
exports.wasmFolder = wasmFolder;
exports.workerErrors = workerErrors;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=blinkcard-sdk.js.map
